!===========================================================================
!===========================================================================
MODULE FLSA0300_SURFACE_ALBEDO
	implicit none
	integer nrec,nu0,npw,no3,nta,nspec,ntyp,ncoef,nv

	parameter(nu0=10,npw=7,no3=4,nta=6,nspec=5,ntyp=3)
	parameter(nrec=nu0*npw*no3*nta*nspec*ntyp)
	parameter(ncoef=3,nv=6)
	real coefs(ncoef,nrec)
	logical ichoose(nv,2**nv) 
	integer iws_loaded
	save iws_loaded , coefs,ichoose
	data iws_loaded /0/
	
CONTAINS
!--------------------------------------------------------------
	subroutine flsa0300_lut_loader(ifile)
	implicit none
	integer iunit_flsa_lut,ifile
	character*21 flsa_lut_name(1)
	data iunit_flsa_lut /20/
	data flsa_lut_name &
        /'flsa0300_lut.2s.coef'/
	
	 print*, 'Load FLSA file :' ,flsa_lut_name(ifile)

	 open(iunit_flsa_lut,file = flsa_lut_name(ifile), &
         form='unformatted',status='old',access='direct',recl=ncoef*nrec*4)

	 read (iunit_flsa_lut,rec=1) coefs
	 close(iunit_flsa_lut)
	 
	 call tf_table(nv,2**nv,ichoose)

	 iws_loaded = ifile
	return
	end subroutine flsa0300_lut_loader
!--------------------------------------------------------------
!--------------------------------------------------------------
	real function flsa0300_lut(toa,u0,pw,o3,ta,spec,ityp,psfc)
	implicit none
	
	real toa,u0,pw,o3,ta,spec,psfc
	integer ityp
	real rlogmin,rnode

	integer iv
!	integer ifile
	integer nva(nv),iva(nv)
	real vmm(2,nv),rng(nv),val(nv),rva(nv)

	parameter(rlogmin=1.0E-10)

	data nva/nu0,npw,no3,nta,nspec,ntyp/
	data vmm/ 0.1 , 1.0, &
     		   -4 ,   2, &
     		   200, 500, &
                   -5,    0, &
                   -1,    1, &
		    1,	  3 /
! ---
!	if( ifile .ne. iws_loaded ) call flsa_lut_loader(ifile)
! ---
	val(1:nv)=vmm(1,1:nv)
	val(1)=u0
	if(pw >= rlogmin) val(2)= log(pw) 
	val(3)= o3
	if(ta >= rlogmin) val(4)= log(ta)
	
	val(5)= spec
	val(6)= ityp

	do iv=1,nv
	if (val(iv) < vmm(1,iv) ) val(iv) = vmm(1,iv)
	if (val(iv) > vmm(2,iv) ) val(iv) = vmm(2,iv)

	rng = vmm(2,iv)-vmm(1,iv)
	rnode=  (val(iv)-vmm(1,iv)) /rng(iv)
	rva(iv) = 1+ rnode*(nva(iv)-1)
	iva(iv) = nint(rva(iv))
	enddo

	flsa0300_lut = rlui(toa,nva,nv,rva, coefs,ncoef,nrec,psfc)
	 
	return
	end function flsa0300_lut
!-----------------------------------------------------------
	subroutine tf_table(n1,n2,ib)
	implicit none
	integer n1,n2,i,ii
	logical ib(n1,n2)
	do i=1,n2
	do ii=1,n1
	ib(ii,i)=btest(i-1,ii-1)
	enddo
!	print*,i,ib(1:n1,i)
	enddo
	return
	end subroutine tf_table
!----------------------------------------------------------------
	real function rlui(toa,nva,nv,rva, coefs,ncoef,nrec,psfc)
	implicit none
	real toa,swx,top,sfc_toa,x,psfc
	integer nv,ncoef,nrec,i,j,ik,k
	integer  nva(nv),iva(nv,2**nv)
	real wx
	
	real coefs(ncoef,nrec)
	real wb(nv),wt(nv),rva(nv)

	integer ibase(nv),itop(nv)
	
	 ibase = floor(rva)
	 where( ibase < 1) ibase=1

	 itop= ceiling(rva)
	 where( itop > nva) itop=nva

	wb=1.0-(rva-ibase)
	wt=1.0-wb
	
!	print'(A10,10f5.1)','Exact',rva(1:nv)
!	print'(A10,10I5)','BASE',ibase
!	print'(A10,10I5)','TOP',itop
!	print'(A10,10f5.1)','Wb',wb	
!	print'(A10,10f5.1)','Wt',wt

	swx=0
	rlui =0
	ik=0

	NODE : do j=1,2**nv
	wx=1.0

	 DIMENSION: do i=1,nv

	 if( ichoose(i,j) ) then
	  iva(i,j) = itop(i)
	  wx=wx*wt(i)
	 else
	  iva(i,j) = ibase(i)
	  wx=wx*wb(i)
	 endif

	 enddo DIMENSION

	 if( wx > 1.0E-10 ) then
	 ik=ik+1
	 k=ilui(nva,nv,iva(1,j))

	 sfc_toa=  coefs(1,k) + coefs(2,k)*toa + coefs(3,k)*toa*toa
	 x = toa + sfc_toa *(psfc/1013.25)
	 swx       = swx  +     wx
	 rlui      = rlui + x * wx

!	print'(2I4,6I3,I10,4f10.5)',ik,j,iva(1:nv,j),k,x,wx,swx,rlui
	if (swx >= .9999)exit
	endif

	enddo NODE
	rlui = rlui/swx
	return
	end function rlui
!-----------------------------------------------------------------------
	integer function ilui(nva,nv,iva)
	implicit none
	integer nv,i,k
	integer  nva(nv),iva(nv)
	integer  mva(nv)

	mva(1)=nva(1)
	do i=2,nv
	 mva(i)=mva(i-1)*nva(i)
	enddo

	k= iva(1)
	do i=2,nv
	k= k+ (iva(i)-1)*mva(i-1)
	enddo

	ilui=k
	return
	end function ilui
END MODULE FLSA0300_SURFACE_ALBEDO
