	module hu_cox_munk_ocean_albedo
	real dat(21,20,15,15)
	contains
!===================================================
	subroutine load_hu_cox_munk_ocean_albedo
!---------------------------------------------------
	iunit=10

	open(iunit,file='hu_cox_munk.ocean_albedo.2'
c     &  ,form='U',access='D',recl=21*20*15*15*4)
     &  ,form='unformatted',access='direct'
     1  ,recl=21*20*15*15*4)

	read(iunit,rec=1) dat

	close(iunit)
	end subroutine load_hu_cox_munk_ocean_albedo
!===================================================
	real function get_hu_cox_munk_ocean_albedo
     & (u0,tau,wnds,ib)
! Inputs:
!u0 : Cosine Solar Zenith Angle (0-1.0)
!tau: Aerosol &/or Cloud Optical Depth
!Wnds: windspeed [m sec-1]
! ib : Fu-liou subband [1-10] 
!     :  11 = Fuliou band 2
!     :  12 = Fuliou band 3
!     :  13 = Fuliou band 4
!     :  14 = Fuliou band 5
!     :  15 = Fuliou band 6
! Output:
!   Spectral Ocean Surface Albedo[0-1]


	real ,save :: u0x(20),taus(21),wndspd(15)



!Fu-liou subband/band (ib)
!1-15

!Windspeed (iw)
!1-15
 	data wndspd /0.1,2,4,6,8,10,12,14,16,18,20,25,30,35,40/
	iw=-1
	if( wnds <= 0.1 ) iw=1
	if( wnds >= 40. ) iw=15
	do i =1,14
	if( wnds > wndspd(i).and. wnds <= wndspd(i+1) ) iw=i
	if( iw > 0 ) exit
	enddo

! Aerosol &/or Cloud Tau (it)
!1-21


	data taus  / 0.0 , 0.10 ,0.133, 0.176,0.235,0.312,
     & 0.416,0.553,0.736,0.979,1.30,1.73,2.30,3.06,4.07,
     & 5.41,7.21,9.58,12.74,16.9,22.5 /

	tauf = tau

	it=-1
	if ( tauf  < taus(1)  ) it= 1
	if ( tauf >= taus(21) ) it=21
	do i = 1,20
	 if (tauf >= taus(i) .and. tauf < taus(i+1) ) it = i 
	 if( it >0 ) exit
	enddo

!	if( tauf >0)  it= 2.5 +(( log(tauf)+2.30 )/0.285 ) 
	


!Cosine Solar Zenith (iu) 
!1-20
	 data u0x /
     & 3.44E-03,1.801E-02,4.388E-02,8.044E-02,
     & 0.1268,0.18197,0.24457,0.31315,0.38611,
     & 0.46174,0.53826,0.61389,0.68685,0.75543,
     & 0.818030,0.87317,0.91956,0.95612,0.98199,0.99656/

	iu=-1
	if (u0 < u0x(1) ) iu= 1
	if (u0 >= u0x(20) ) iu= 20
	do i = 1,19
	 if (u0 >= u0x(i) .and. u0 < u0x(i+1) ) iu = i 
	 if( iu >0 ) exit
	enddo

	if( iu == 20 .or. it==21) then
	 salb=  dat(it,iu,iw,ib)
	else
	 salb1 =  dat(it,iu,iw,ib)
	 salb2 =  dat(it,iu+1,iw,ib)
	 salb11=  dat(it+1,iu,iw,ib)
	 salb22=  dat(it+1,iu+1,iw,ib)

	
	f0=  u0x(iu+1)-u0x(iu) 
	f1=  u0x(iu+1)-u0
	f2=  u0 - u0x(iu)

	f00=  taus(it+1)-taus(it) 
	f11=  taus(it+1) - tauf
	f22=  tauf - taus(it) 

	salb= ( (f1/f0)*salb1  + (f2/f0)*salb2  ) *(f11/f00) +
     &	      ( (f1/f0)*salb11 + (f2/f0)*salb22 ) *(f22/f00)

	endif

	get_hu_cox_munk_ocean_albedo = salb 

!	print'(3(f7.2,i4),i4,3f8.4)'
!     &   ,tauf,it,u0,iu,wnds,iw,ib,
!     &   get_hu_cox_munk_ocean_albedo
!	write(31)wnds,tauf,u0,salb


	end function get_hu_cox_munk_ocean_albedo



!===================================================
	real function ocean_foam_alb(wnds,Tair,Th2o,albin)
!INPUTS::
! wnds - wind speed [meter sec-1]
! Tair - Air Temperture [K]
! Th2o - Water Temperature [K]
! albin - Surface Albedo before Foam Correction.

!! Underlight correction
	 ulc  = 0.005 * (1.-albin )
	 alb2 = albin +ulc
!!! FOAM Correction
	 f = 1.951e-5 * wnds**2.55 * exp(0.086*(Th2o-Tair))
	 ocean_foam_alb = 0.3*f +(1-f)*alb2

	return
	end function ocean_foam_alb

!===================================================

	end module hu_cox_munk_ocean_albedo
!===================================================
