	MODULE SPEC_SFC

	USE SPECTRAL_DAT

	USE SFCALB_HISTORY ,
     &   ONLY :OVERLAY_MAPS, USE_SFCALB_HISTORY ,DICKENSONX,
     &   map_index,ivector,igbpa

	USE HU_COX_MUNK_OCEAN_ALBEDO ,
     &   ONLY:LOAD_HU_COX_MUNK_OCEAN_ALBEDO,
     &       GET_HU_COX_MUNK_OCEAN_ALBEDO,
     &       OCEAN_FOAM_ALB

       USE FLSA0300_SURFACE_ALBEDO,
     &  ONLY:FLSA0300_LUT_LOADER,FLSA0300_LUT
!------------------------------------------------------------
! INPUTS ::
!  rlat - Sfc Latitude
!  rlon - SFC Longitude
!  obs_alb - CERES Observed SW TOA Albedo (0-1)
!  u0 - Cosine Solar Zenith Angle (0-1)
!  sfc_type(8) - SSF vector of IGBP SCENE Type (1-20)
!  sfc_area(8) - SSF vector of Area Coverage% of IGBP type (0-100)
!  area_clr - SSF Clear area% of Footprint (0-100)
!  area_cld(2) - Area% of each cloud condition (0-100)
!  optdepth_mn_log(2) - Log of cloud optical Depth each cld cnd. 
!  precip_h2o - SSF total collumn PW (cm)
!  sfcwind - Surface windspeed (msec-1) ! derived from SSF u&v components
!  aerosol_tau - Aerosol Optical Depth (visible)
!  o3 - Collumn Ozone ammount (DU)
!  tair - Surface air temperature (K)
!  tskin - Skin Temp  (K)
!  YEAR - char*4 eg. '1998'
!  MONTH -char*2 eg. '01'  
! OUTPUT:
! specalb(15,0:2) - 15 band spectral surface albedo for fu-liou code
!                   in bands (1-10)-->as(1:10)   
!		    bands 11-15 as(2-6)
!		    cloud conditions 0:2 , 0=clear 1:2 cloud cond.
! bba(0:2) - broad band albedo ,cloud conditions 0:2 , 0=clear 1:2 cloud cond.
! specalb0(15) - Cloud area weighted Spectral albedo ( FOV MEAN SPECTRAL SFC ALBEDO)
! bba0 - Cloud area weighted  broadband albedo ( FOV MEAN SFC ALBEDO )
!
! emiss(12) - spectral surface emissivity in 12 LW fu-liou bands
!==============================================================================
	TYPE spectral_surface
	sequence
!Input...
	real*4  rlat		! Latitude of FOV
	real*4  rlon		! Longitude of FOV
	real*4  obs_alb 	! OBSERVED CERES TOA ALBEDO
	real*4    u0   		! Cosine Solar Zenith Angle
	integer*2  sfc_type(8)
	integer*2  sfc_area(8)
	integer*2  area_clr
	integer*2  area_clds(2)
	real*4    optdepth_mn_log(2) 
	real*4    precip_h2o
	real*4    sfcwind
        real*4    sfc_alt
	real*4    aerosol_tau
	real*4    o3
	real*4    tair
	real*4    tskin
	character*4 YEAR
	character*2 MONTH

! Output ...
	real*4    specalb(15,0:2)
	real*4    bba(0:2)
	real*4    specalb0(15)
	real*4    bba0
	real*4    emiss(12)

!Diagnostic
	integer igbp1 ! Predominate sfc type of FOV
	integer ico   ! Case Flag ( 1=ocean 2=clrland 3=cldland 4=clrsnow 5=cldsnow 6=seaice )
	END TYPE

	type(spectral_surface) ::ss

	contains
!=============================================================
	subroutine choose_spectral_properties

!Begin ....
	ss%ico=-9999. !! CASE output unit
100	continue 
	where( ss%sfc_type > 21 ) ss%sfc_type =0
	where( ss%sfc_area > 101 ) ss%sfc_area =0
	ss%igbp1=  maxval(ss%sfc_type( maxloc( ss%sfc_area)))

	call sarb_cookie_emiss
!!!	print'(I4,12f10.3)',ss%igbp1,ss%emiss(1:12)

	ss%specalb=0.0 ; ss%specalb0=0.0
	ss%bba    =0.0 ; ss%bba0    =0.0

	if ( ss%igbp1 == 17 ) then ! OCEAN (clr&cld)	
	  call as_ocean ;ss%ico =1

	elseif ( ss%igbp1 <= 18 ) then! Land

	 call sarb_cookie_albedo	

	   if ( ss%area_clr == 100 ) then 
	  
	    call as_clr_land ; ss%ico=2

	       if ( ss%bba0 > 0.7 .and. ss%u0 > 0.2 .and. ss%igbp1.ne.19) then !!! reset IGBP to snow
	        ss%sfc_type(1) = 19 
		ss%sfc_area(1) = 100 ; ss%sfc_area(2:8)=0
	        go to 100 !! Do Again Assume snow spectral Albedo
	       endif 

	   else

	     call as_cld_land ; ss%ico =3

	   endif


	elseif(  ss%igbp1 == 19 ) then ! Snow
	 call snow_over
	 call sarb_cookie_albedo ; ss%ico=5
	   if ( ss%area_clr == 100 ) then
	    call as_clr_land ; ss%ico=4
	   endif

	elseif ( ss%igbp1 == 20 ) then ! SEA ICE	(clr&cld)
	 call sarb_cookie_albedo ; ss%ico =6



	else 
	STOP ' bad igbp'
	endif


	return
	end subroutine choose_spectral_properties

!============================================================================
	subroutine snow_over
!--------------------------------------
! weights for various scenes under snow
!  1 - grassland etc...
!  2 - Deciduous Forest
!  3 - Evergreen Needleleaf
!  4 - Mixed Deciduous
!--------------------------------------
	integer*2 area2(4)
        data area2/14,86,92,96/

	ii = map_index(ss%rlat,ss%rlon)
	is = ivector(ii)
	rigbpmap = igbpa(is) !! RAW IGBP MAp

	 it2=1
	 if ( rigbpmap == 2 .or. rigbpmap == 3 ) it2=2 
	 if ( rigbpmap == 1  ) it2=3 
	 if ( rigbpmap == 5  ) it2=4
 
	 ss%sfc_type(1) = 19 
	 ss%sfc_area(1) = 100-area2(it2)
	 ss%sfc_type(2) = it2 
	 ss%sfc_area(2) = area2(it2) 
         ss%sfc_area(3:8)=0
	areasnow=ss%sfc_area(1)
	
	return
	end subroutine snow_over
!====================================================================================
	subroutine sarb_cookie_emiss
	real ems12(12)
	ss%emiss=0.0
	TYPES: do i=1,8
	 area_igbp= ss%sfc_area(i)
	 if (area_igbp == 0 ) cycle
	 igbp= ss%sfc_type(i)

	 call get_emiss(igbp,ems12)
	 ss%emiss(1:12)= ss%emiss(1:12) + ems12(1:12) *(area_igbp/100.)
	enddo TYPES
	return

	end subroutine sarb_cookie_emiss
!=====================================================================================
	subroutine sarb_cookie_albedo
! Weight IGBP based Spectral Albedos by PSF footprint IGBP Type
! Move By diurnal "d" Model to U0 ( clear) , Diffuse(Cloudy)
! Seperate Spectral Albedo, BBalb for Clear & Cloudy.
! Mean Over Footprint also provided. 

        real salb15(15),bbalb
	

	  ss%specalb0=0.0 ;ss%bba0 =0.0
	  ss%specalb =0.0 ;ss%bba  =0.0

	CLOUDC : do icldc=0,2
 	if (icldc == 0)then
	  area_cnd = ss%area_clr
	  u0x= ss%u0
	 elseif( icldc >0 .and. ss%area_clds(icldc) >0 )then
	  area_cnd = ss%area_clds(icldc)
	  u0x=0.5 !! DIFFUSE 
	 else
	  cycle
	 endif
	
	TYPES: do i=1,8
	area_igbp= ss%sfc_area(i)
	if (area_igbp == 0 ) cycle
	igbp= ss%sfc_type(i)

	call land_spec(igbp,u0x, salb15,bbalb)

!----------------------------------------------------------
!  adjust snow reflectivity according to surface temp.
!----------------------------------------------------------
	if ( igbp == 19 .and. ss%tskin > 274 ) then
	 salb15 = salb15*0.74 
	 bbalb  = bbalb*0.74
	endif

	ss%bba(icldc)          = ss%bba(icldc)          
     1                         + bbalb      *(area_igbp/100.)
	ss%specalb(1:15,icldc) = ss%specalb(1:15,icldc) 
     1                         + salb15(1:15)*(area_igbp/100.)
	
	enddo TYPES

	ss%bba0          =ss%bba0         
     1                  + ss%bba(icldc)         *(area_cnd/100.)
	ss%specalb0(1:15)=ss%specalb0(1:15)
     1                  + ss%specalb(1:15,icldc)*(area_cnd/100.)

	enddo CLOUDC

	return

	end subroutine sarb_cookie_albedo
!=====================================================================================
	subroutine adj_spect_shape(sfcalb0,sfcalb1)

	  ss%specalb0=0.0 ;ss%bba0 =0.0

	CLOUDC : do icldc=0,2
 	if (icldc == 0)then
	  area_cnd = ss%area_clr
	  ss%specalb(1:15,0)=ss%specalb(1:15,0)* (sfcalb0/ss%bba(0) )
	  ss%bba(0) = sfcalb0
	 elseif( icldc >0 .and. ss%area_clds(icldc) >0 )then
	  area_cnd = ss%area_clds(icldc)
	  ss%specalb(1:15,icldc)=ss%specalb(1:15,icldc)* 
     1                            (sfcalb1/ss%bba(icldc) )
	  ss%bba(icldc) = sfcalb1
	 else
	 ss%specalb(1:15,icldc)=-9999.
	 ss%bba(icldc)         =-9999.
	  cycle
	 endif

	  ss%specalb0(1:15)=ss%specalb0(1:15)+ 
     1                      ss%specalb(1:15,icldc)*(area_cnd/100.) 
	  ss%bba0 = ss%bba0 + ss%bba(icldc)*(area_cnd/100.)
	enddo CLOUDC 

	where ( ss%bba > 1.0     ) ss%bba = 1.0
	where ( ss%bba < 0.0   .and.  ss%bba .ne. -9999.   ) ss%bba = 0.0
	where ( ss%specalb > 1.0  ) ss%specalb = 1.0
	where ( ss%specalb < 0.0 .and. ss%specalb .ne. -9999. ) ss%specalb = 0.0
	if    ( ss%bba0 > 1.0     ) ss%bba0 = 1.0
	if    ( ss%bba0 < 0.0   .and. ss%bba0 .ne. -9999. ) ss%bba0 = 0.0
	where ( ss%specalb0 > 1.0 ) ss%specalb0 = 1.0
	where ( ss%specalb0 < 0.0 .and. ss%specalb0 .ne. -9999.) 
     1         ss%specalb0 = 0.0


	return

	end subroutine adj_spect_shape
!=====================================================================================
	subroutine as_clr_land
! FLSA

	spec  =  specshape( ss%igbp1 )        !!! based on IGBP lookup !! SSF SFCTYPE 1-20
	iatype=  2 			   !!! Continental Aerosol
	psfc  =  1013.*exp(-ss%sfc_alt/8000.)!!! Surface Pressure based on SSF Sfcaltitude(meters)

	sfcalb0= flsa0300_lut(ss%obs_alb,ss%u0,ss%precip_h2o,ss%o3,
     &                           ss%aerosol_tau,spec,iatype,psfc)

	if  ( sfcalb0 < 0.0 ) then ! Bad retrieval use Sfcalb History Value
	 call as_cld_land 
	 return
	endif


	call adj_spect_shape(sfcalb0,sfcalb0)


	return
	end subroutine as_clr_land

!=====================================================================================
	subroutine as_cld_land
! SFCALBHIST
	call use_sfcalb_history(ss%rlat,ss%rlon,ss%u0, icount,sfcalb0)
	
	if(sfcalb0 <0) then ! If not on land map, call it ocean...
	        call as_ocean 
	         ss%igbp1 = 17 
	         ss%sfc_type(1) = 17  ! ocean  
		 ss%sfc_area(1) = 100 ; ss%sfc_area(2:8)=0
	        return
	endif

	dd= d(ss%igbp1) !! D of most prevalent scene type.
	u0x=0.5 

 	sfcalb1 = dickensonx(sfcalb0,dd,ss%u0,u0x)  !! Cloudy Diffuse angle
	
	call adj_spect_shape(sfcalb0,sfcalb1)

	return
	end subroutine as_cld_land
!=====================================================================================
	subroutine as_ocean
! HU_COX_MUNK

	ss%bba0 =0.0 ;ss%specalb0 =0.0 

	 CLOUD_CONDITION : do icldc=0,2 
	 if (icldc == 0)then
	  tau  = ss%aerosol_tau
	  area = ss%area_clr
	 elseif( icldc >0 .and. ss%area_clds(icldc) >0 )then
	  area = ss%area_clds(icldc)
	  tau  = ss%aerosol_tau+exp(ss%optdepth_mn_log(icldc))
	 else
	 ss%specalb(1:15,icldc)=-9999.
	 ss%bba(icldc)         =-9999.
	  cycle
	 endif

	SPECTRAL: do ib=1,15
!! LOOKUP OCEAN SPECTRAL ALBEDO FOR Fu-Liou Band! Hu-Cox-Munk 
	talb = get_hu_cox_munk_ocean_albedo(ss%u0,tau,ss%sfcwind,ib)
!!! Correct for Ocean Foam & Underlight
	ss%specalb(ib,icldc)=ocean_foam_alb(ss%sfcwind,ss%tair,ss%tskin,talb)
	enddo SPECTRAL
	ss%specalb0 =ss%specalb0 + ss%specalb(1:15,icldc)*(area/100.)

	ss%bba(icldc)=sum(ss%specalb(1:15,icldc)*wgts(1:15))
	ss%bba0 = ss%bba0+ss%bba(icldc)*(area/100.)

	
	enddo CLOUD_CONDITION


	return
	end subroutine as_ocean


	end module SPEC_SFC
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	Program test_spec_sfc

	USE SPEC_SFC

! INITALIZE LOAD TABLES
	call flsa0300_lut_loader(1)
	call load_hu_cox_munk_ocean_albedo
	ss%YEAR = '1998'
	ss%MONTH = '01'
	call overlay_maps(ss%YEAR,ss%MONTH,'4')

	open(10,file='/usr3/rose/ceres/crs/crs.21v6.a.3.day',
     1       form='unformatted')
	icnt=0
	FOV : do 
	read(10,end=999)ss%rlat,ss%rlon,ss%obs_alb,		
     &  ss%sfc_type,ss%sfc_area, 		
     &  ss%area_clr,ss%area_clds(1:2), 
     &	ss%optdepth_mn_log(1:2),
     &	ss%precip_h2o,sfcwind_u,sfcwind_v,		
     &	szen_toa,ss%sfc_alt,ca_optdepth063_clr
	if (ss%obs_alb < 0)cycle
	icnt=icnt+1
	if ( mod(icnt,100000) .eq.0) print*,icnt
	  if( icnt == 1000001 ) stop
! Compute Cossol
	ss%u0   = cos( szen_toa*3.14159/180.)
! Assign SFCAIR & SKIN Temperatures FROM MOA
	ss%tair =300  !!! MOA SFC AIR
	ss%tskin =300 !!! Get from MOA

! Assign Collumn OZONE from MOA
	ss%o3   =300. !!! MOA Collum Ozone  ! constant Value
! Assign Aerosol from Current source ( STOWE, GFDL , Collins , Modis ...???)	
	ss%aerosol_tau=ca_optdepth063_clr
	if ( ss%aerosol_tau > 3.0 .or. ss%aerosol_tau < 0)  ss%aerosol_tau=0.1
	
! Compute sfc windspeed from SSF u & v components
	ss%sfcwind = 5.0 !meters sec-1 
	if ( sfcwind_u < 1000 .and. sfcwind_v < 1000 )
     &	 ss%sfcwind = sqrt( sfcwind_u**2+ sfcwind_v**2)

!FAKE OUT for testing -----------------------------------------
	if ( ss%rlat > 30.) then 
	 ss%tair =270  !!! MOA SFC AIR
	 ss%tskin =270 !!! Get from MOA
	endif

	where( ss%sfc_type > 21 ) ss%sfc_type =0
	where( ss%sfc_area > 101 ) ss%sfc_area =0
	ss%igbp1=  maxval(ss%sfc_type( maxloc( ss%sfc_area)))
	if (ss%rlat > 20 .and. ss%igbp1 .ne. 17 ) then
	 ss%sfc_type(1) = 19 
	 ss%sfc_area(1) = 100 ; ss%sfc_area(2:8)=0
	 ss%igbp1=  maxval(ss%sfc_type( maxloc( ss%sfc_area)))
	elseif (ss%rlat > 20 .and. ss%igbp1 .eq. 17 ) then
	 ss%sfc_type(1) = 20 
	 ss%sfc_area(1) = 100 ; ss%sfc_area(2:8)=0
	 ss%igbp1=  maxval(ss%sfc_type( maxloc( ss%sfc_area)))
	endif
!FAKE OUT for testing-----------------------------------------

	    call choose_spectral_properties
	   
!--- Write out ----------------------------------------------------------
	if (ss%ico > 0 )then
!mean over FOV
!	print'(2I4,2f6.0,f6.2,15f7.3,2x,f7.3)',ss%igbp1,-9999,-9999.,-9999.,
!     & ss%u0,ss%specalb0(1:15),ss%bba0
!	print*,' '

	aigbp=ss%igbp1
	aico=ss%ico
	aclear=ss%area_clr !! I2-->R4
	write(30+ss%ico)ss%rlat,ss%rlon,
     & ss%u0,ss%bba0,aclear,ss%optdepth_mn_log(1),ss%sfcwind,ss%bba(0:1)
	write(50)ss%rlat,ss%rlon,
     & ss%u0,ss%bba0,aclear,ss%optdepth_mn_log(1),ss%sfcwind,ss%bba(0:1)
     &  ,aigbp,aico
	else
	 stop' bad ico'
	endif
!------------------------------------------------------------------------

	enddo FOV

999	stop
	end
