
!!===================================================================
	subroutine add_chou3(np,co2in,u0,pl,tl,ql,dn,up,dn1,up1,npc
     & ,ntop,nbot)
	
	real,dimension(np+1) :: pl,tl,ql,dn,up, dn1,up1
	real,dimension(np+1) :: df,dfu,dff,adj_B,valb0,valb1
	real,dimension(np+1) :: df_cld, dfu_cld
	real,dimension(np+1) :: df_diff, dfu_diff

	u0d=0.5 !! DIFFUSE ANGLE
	valb0=up/dn    ! Albedo at level
	
	co2=co2in/1.0E+6 !! ppmv to PPV
	 df=0 ; dfu=0 ; df_cld=0 ; dfu_cld=0 ; df_diff=0 ; dfu_diff=0

	 call chou_o2(np,pl,u0,df,dfu)
	 call chou_co2(np,pl,tl,ql,u0,co2,df,dfu)

!! Based on Fraction of TOA DN insolation ->> decrease on Flux
	 dff= df*dn(1) 

	if (npc >0 .and. npc .le. np .and. ntop .le. npc) then
	
	 call chou_o2(npc,pl,u0,df_cld,dfu_cld)
 	 call chou_co2(npc,pl,tl,ql,u0,co2,df_cld,dfu_cld)

	 call chou_o2(np,pl,u0d,df_diff,dfu_diff)
 	 call chou_co2(np,pl,tl,ql,u0d,co2,df_diff,dfu_diff)

!! Based on Fraction of TOA DN Just above cloud insolation ->> decrease on Flux
!	 dff(npc+1:np+1)= df_diff(npc+1:np+1)*dn(1)* dn(npc+1:np+1)/dn(npc-1)  
	 dff(ntop+1:npc)=      df(ntop+1:npc)*  dn(ntop+1:npc)
	 dff(npc+1:np+1)= df_diff(npc+1:np+1)*  dn(npc+1:np+1)
	endif


! Adjustment for Downward Flux	
	dn1		=   dn - dff
! Adjustment for Sfc Reflected due to Absorption in Downward

	adj_A = ( dn(np+1)-dn1(np+1) ) *valb0(np+1)

! Adjustment for  Absorption in Upward as Fraction of Surface Reflected

	adj_B(1:np+1)   = adj_A    !! ~ SFC REFLECTED
 	adj_B(1:np)     = adj_B(1:np) +
     &                    (dfu(1:np)-df(np+1))* up(np+1)  ! scaled to sfcup

	if (npc >0 .and. npc .le. np) then !! adjustment ~ SFC + CLOUD REFLECTED
	adj_A = ( dn(npc)-dn1(npc) ) *valb0(npc)
! Adjust A NOW based upon mean cloud albedo Not at defined cloud level
!	adj_A = ( dn(npc)-dn1(npc) )*( valb0(ntop)+valb0(nbot) )*0.5

! To derive a rough estimate of how much energy makes its way
! to the top of the cloud from the surface as opposed to being reflected from
! the defined cloud level(npc)	
	clddiv= (dn(ntop)-up(ntop) ) - (dn(nbot)-up(nbot) )
	r2 =  (up(ntop)+up(nbot) ) / (dn(ntop)+dn(nbot) ) 
	upcldabs = r2*clddiv
	r1=0
	if(up(ntop)>0) r1= (up(nbot)-upcldabs) /up(ntop)
	if(r1 < 0) r1=0
	if(r1 > 1) r1=1.0
	print'(a4,3I4,4f8.3)','CPRF',npc,ntop,nbot,clddiv,upcldabs,r1
	adj_B(1:npc)   =  adj_A   !! ~ CLD REFLECTED
        adj_B(1:npc-1)= adj_B(1:npc-1)+
     &          r1 *(dfu_cld(1:npc-1)-df_cld(npc) )*up(npc)  !scaled to cldup, Cloud Path
     &   + (1.0-r1)*(    dfu(1:npc-1)-    df(np+1))*up(np+1) !scaled to sfcup, Surface Path
	endif

	up1            =  up   - adj_B

	valb1=up1/dn1 ! Adjusted  Albedos
	if ( abs( valb0(np+1)-valb1(np+1)) > 0.001) stop
     &  ' CHOU SURFACE ALB ERR'

	return
	end

!=================================================================

!----------------------------------------------------------------
	subroutine chou_co2(np,pl,tl,ql,u0,co2,df,dfu)
	implicit none
	integer i,j,k,np
	real u0,u0d,co2
	real ,dimension(np+1)::pl,tl,ql,so2d,so2u,
     &                         swud,swhd,swuu,swhu,df,dfu
	real ,dimension(np) :: dp   , scal ,wh ,wa ,ta

c-----parameters for co2 transmission tables

      integer nu,nw,nx,ny
      parameter (nu=43,nw=37,nx=62,ny=101)
      real, save:: w1,dw,u1,du,coa(nx,ny),cah(nu,nw)

c-----cah is the co2 absorptance in band 10

!!       include "cah.data"
      data ((cah(i,j),i=1,43),j=  1,  1)/
     &  0.0000001,  0.0000001,  0.0000001,  0.0000002,  0.0000002,
     &  0.0000003,  0.0000005,  0.0000007,  0.0000009,  0.0000013,
     &  0.0000019,  0.0000026,  0.0000037,  0.0000053,  0.0000074,
     &  0.0000104,  0.0000147,  0.0000206,  0.0000288,  0.0000402,
     &  0.0000559,  0.0000772,  0.0001059,  0.0001439,  0.0001936,
     &  0.0002575,  0.0003384,  0.0004400,  0.0005662,  0.0007219,
     &  0.0009131,  0.0011470,  0.0014327,  0.0017806,  0.0022021,
     &  0.0027093,  0.0033141,  0.0040280,  0.0048609,  0.0058217,
     &  0.0069177,  0.0081559,  0.0095430/
      data ((cah(i,j),i=1,43),j=  2,  2)/
     &  0.0000001,  0.0000001,  0.0000001,  0.0000002,  0.0000002,
     &  0.0000003,  0.0000005,  0.0000007,  0.0000009,  0.0000013,
     &  0.0000019,  0.0000026,  0.0000037,  0.0000053,  0.0000074,
     &  0.0000104,  0.0000147,  0.0000206,  0.0000288,  0.0000402,
     &  0.0000559,  0.0000772,  0.0001059,  0.0001439,  0.0001936,
     &  0.0002575,  0.0003384,  0.0004400,  0.0005662,  0.0007219,
     &  0.0009130,  0.0011470,  0.0014326,  0.0017805,  0.0022020,
     &  0.0027091,  0.0033139,  0.0040276,  0.0048605,  0.0058211,
     &  0.0069170,  0.0081551,  0.0095420/
      data ((cah(i,j),i=1,43),j=  3,  3)/
     &  0.0000001,  0.0000001,  0.0000001,  0.0000002,  0.0000002,
     &  0.0000003,  0.0000005,  0.0000007,  0.0000009,  0.0000013,
     &  0.0000019,  0.0000026,  0.0000037,  0.0000053,  0.0000074,
     &  0.0000104,  0.0000147,  0.0000206,  0.0000288,  0.0000402,
     &  0.0000559,  0.0000772,  0.0001059,  0.0001439,  0.0001936,
     &  0.0002574,  0.0003384,  0.0004399,  0.0005661,  0.0007218,
     &  0.0009129,  0.0011468,  0.0014325,  0.0017803,  0.0022017,
     &  0.0027088,  0.0033135,  0.0040271,  0.0048599,  0.0058204,
     &  0.0069161,  0.0081539,  0.0095406/
      data ((cah(i,j),i=1,43),j=  4,  4)/
     &  0.0000001,  0.0000001,  0.0000001,  0.0000002,  0.0000002,
     &  0.0000003,  0.0000005,  0.0000007,  0.0000009,  0.0000013,
     &  0.0000019,  0.0000026,  0.0000037,  0.0000053,  0.0000074,
     &  0.0000104,  0.0000147,  0.0000206,  0.0000288,  0.0000402,
     &  0.0000559,  0.0000772,  0.0001059,  0.0001439,  0.0001936,
     &  0.0002574,  0.0003384,  0.0004399,  0.0005661,  0.0007217,
     &  0.0009128,  0.0011467,  0.0014323,  0.0017800,  0.0022014,
     &  0.0027084,  0.0033130,  0.0040265,  0.0048591,  0.0058194,
     &  0.0069148,  0.0081524,  0.0095387/
      data ((cah(i,j),i=1,43),j=  5,  5)/
     &  0.0000001,  0.0000001,  0.0000001,  0.0000002,  0.0000002,
     &  0.0000003,  0.0000005,  0.0000007,  0.0000009,  0.0000013,
     &  0.0000019,  0.0000026,  0.0000037,  0.0000053,  0.0000074,
     &  0.0000104,  0.0000147,  0.0000206,  0.0000288,  0.0000402,
     &  0.0000559,  0.0000772,  0.0001059,  0.0001439,  0.0001935,
     &  0.0002574,  0.0003383,  0.0004398,  0.0005660,  0.0007216,
     &  0.0009127,  0.0011465,  0.0014320,  0.0017797,  0.0022010,
     &  0.0027078,  0.0033123,  0.0040256,  0.0048580,  0.0058180,
     &  0.0069132,  0.0081503,  0.0095361/
      data ((cah(i,j),i=1,43),j=  6,  6)/
     &  0.0000001,  0.0000001,  0.0000001,  0.0000002,  0.0000002,
     &  0.0000003,  0.0000005,  0.0000007,  0.0000009,  0.0000013,
     &  0.0000019,  0.0000026,  0.0000037,  0.0000053,  0.0000074,
     &  0.0000104,  0.0000147,  0.0000206,  0.0000288,  0.0000402,
     &  0.0000559,  0.0000772,  0.0001059,  0.0001439,  0.0001935,
     &  0.0002573,  0.0003383,  0.0004398,  0.0005659,  0.0007215,
     &  0.0009125,  0.0011462,  0.0014317,  0.0017792,  0.0022004,
     &  0.0027071,  0.0033113,  0.0040244,  0.0048565,  0.0058162,
     &  0.0069109,  0.0081476,  0.0095328/
      data ((cah(i,j),i=1,43),j=  7,  7)/
     &  0.0000001,  0.0000001,  0.0000001,  0.0000002,  0.0000002,
     &  0.0000003,  0.0000005,  0.0000007,  0.0000009,  0.0000013,
     &  0.0000019,  0.0000026,  0.0000037,  0.0000053,  0.0000074,
     &  0.0000104,  0.0000147,  0.0000206,  0.0000288,  0.0000402,
     &  0.0000559,  0.0000772,  0.0001058,  0.0001438,  0.0001935,
     &  0.0002573,  0.0003382,  0.0004396,  0.0005657,  0.0007213,
     &  0.0009122,  0.0011459,  0.0014312,  0.0017786,  0.0021996,
     &  0.0027061,  0.0033100,  0.0040228,  0.0048545,  0.0058137,
     &  0.0069079,  0.0081439,  0.0095283/
      data ((cah(i,j),i=1,43),j=  8,  8)/
     &  0.0000001,  0.0000001,  0.0000001,  0.0000002,  0.0000002,
     &  0.0000003,  0.0000005,  0.0000007,  0.0000009,  0.0000013,
     &  0.0000019,  0.0000026,  0.0000037,  0.0000052,  0.0000074,
     &  0.0000104,  0.0000146,  0.0000206,  0.0000288,  0.0000402,
     &  0.0000558,  0.0000772,  0.0001058,  0.0001438,  0.0001934,
     &  0.0002572,  0.0003381,  0.0004395,  0.0005655,  0.0007210,
     &  0.0009119,  0.0011454,  0.0014306,  0.0017778,  0.0021985,
     &  0.0027047,  0.0033084,  0.0040207,  0.0048519,  0.0058105,
     &  0.0069040,  0.0081391,  0.0095225/
      data ((cah(i,j),i=1,43),j=  9,  9)/
     &  0.0000001,  0.0000001,  0.0000001,  0.0000002,  0.0000002,
     &  0.0000003,  0.0000005,  0.0000007,  0.0000009,  0.0000013,
     &  0.0000019,  0.0000026,  0.0000037,  0.0000052,  0.0000074,
     &  0.0000104,  0.0000146,  0.0000206,  0.0000288,  0.0000402,
     &  0.0000558,  0.0000771,  0.0001058,  0.0001437,  0.0001933,
     &  0.0002571,  0.0003379,  0.0004393,  0.0005652,  0.0007206,
     &  0.0009114,  0.0011447,  0.0014297,  0.0017767,  0.0021971,
     &  0.0027030,  0.0033061,  0.0040180,  0.0048485,  0.0058064,
     &  0.0068989,  0.0081329,  0.0095149/
      data ((cah(i,j),i=1,43),j= 10, 10)/
     &  0.0000001,  0.0000001,  0.0000001,  0.0000002,  0.0000002,
     &  0.0000003,  0.0000005,  0.0000007,  0.0000009,  0.0000013,
     &  0.0000019,  0.0000026,  0.0000037,  0.0000052,  0.0000074,
     &  0.0000104,  0.0000146,  0.0000205,  0.0000288,  0.0000402,
     &  0.0000558,  0.0000771,  0.0001057,  0.0001437,  0.0001932,
     &  0.0002569,  0.0003377,  0.0004390,  0.0005649,  0.0007201,
     &  0.0009107,  0.0011439,  0.0014286,  0.0017753,  0.0021953,
     &  0.0027006,  0.0033032,  0.0040144,  0.0048441,  0.0058009,
     &  0.0068922,  0.0081248,  0.0095051/
      data ((cah(i,j),i=1,43),j= 11, 11)/
     &  0.0000001,  0.0000001,  0.0000001,  0.0000002,  0.0000002,
     &  0.0000003,  0.0000005,  0.0000007,  0.0000009,  0.0000013,
     &  0.0000019,  0.0000026,  0.0000037,  0.0000052,  0.0000074,
     &  0.0000104,  0.0000146,  0.0000205,  0.0000287,  0.0000401,
     &  0.0000558,  0.0000770,  0.0001056,  0.0001436,  0.0001931,
     &  0.0002567,  0.0003375,  0.0004387,  0.0005644,  0.0007195,
     &  0.0009098,  0.0011428,  0.0014271,  0.0017734,  0.0021929,
     &  0.0026976,  0.0032995,  0.0040097,  0.0048384,  0.0057939,
     &  0.0068837,  0.0081145,  0.0094926/
      data ((cah(i,j),i=1,43),j= 12, 12)/
     &  0.0000001,  0.0000001,  0.0000001,  0.0000002,  0.0000002,
     &  0.0000003,  0.0000005,  0.0000007,  0.0000009,  0.0000013,
     &  0.0000019,  0.0000026,  0.0000037,  0.0000052,  0.0000074,
     &  0.0000104,  0.0000146,  0.0000205,  0.0000287,  0.0000401,
     &  0.0000557,  0.0000770,  0.0001055,  0.0001434,  0.0001929,
     &  0.0002565,  0.0003371,  0.0004382,  0.0005637,  0.0007186,
     &  0.0009087,  0.0011413,  0.0014252,  0.0017709,  0.0021898,
     &  0.0026937,  0.0032946,  0.0040038,  0.0048311,  0.0057850,
     &  0.0068729,  0.0081013,  0.0094768/
      data ((cah(i,j),i=1,43),j= 13, 13)/
     &  0.0000001,  0.0000001,  0.0000001,  0.0000002,  0.0000002,
     &  0.0000003,  0.0000005,  0.0000007,  0.0000009,  0.0000013,
     &  0.0000019,  0.0000026,  0.0000037,  0.0000052,  0.0000074,
     &  0.0000104,  0.0000146,  0.0000205,  0.0000287,  0.0000400,
     &  0.0000556,  0.0000769,  0.0001054,  0.0001432,  0.0001926,
     &  0.0002561,  0.0003366,  0.0004376,  0.0005629,  0.0007175,
     &  0.0009073,  0.0011394,  0.0014228,  0.0017678,  0.0021859,
     &  0.0026888,  0.0032885,  0.0039963,  0.0048218,  0.0057738,
     &  0.0068592,  0.0080849,  0.0094570/
      data ((cah(i,j),i=1,43),j= 14, 14)/
     &  0.0000001,  0.0000001,  0.0000001,  0.0000002,  0.0000002,
     &  0.0000003,  0.0000005,  0.0000007,  0.0000009,  0.0000013,
     &  0.0000019,  0.0000026,  0.0000037,  0.0000052,  0.0000074,
     &  0.0000104,  0.0000146,  0.0000205,  0.0000286,  0.0000400,
     &  0.0000556,  0.0000767,  0.0001052,  0.0001430,  0.0001923,
     &  0.0002557,  0.0003361,  0.0004368,  0.0005619,  0.0007161,
     &  0.0009054,  0.0011370,  0.0014197,  0.0017639,  0.0021809,
     &  0.0026826,  0.0032809,  0.0039869,  0.0048103,  0.0057597,
     &  0.0068422,  0.0080643,  0.0094323/
      data ((cah(i,j),i=1,43),j= 15, 15)/
     &  0.0000001,  0.0000001,  0.0000001,  0.0000002,  0.0000002,
     &  0.0000003,  0.0000005,  0.0000007,  0.0000009,  0.0000013,
     &  0.0000019,  0.0000026,  0.0000037,  0.0000052,  0.0000073,
     &  0.0000103,  0.0000145,  0.0000204,  0.0000286,  0.0000399,
     &  0.0000554,  0.0000766,  0.0001050,  0.0001427,  0.0001919,
     &  0.0002552,  0.0003353,  0.0004358,  0.0005605,  0.0007144,
     &  0.0009032,  0.0011340,  0.0014159,  0.0017590,  0.0021748,
     &  0.0026750,  0.0032715,  0.0039752,  0.0047961,  0.0057424,
     &  0.0068212,  0.0080389,  0.0094019/
      data ((cah(i,j),i=1,43),j= 16, 16)/
     &  0.0000001,  0.0000001,  0.0000001,  0.0000002,  0.0000002,
     &  0.0000003,  0.0000005,  0.0000007,  0.0000009,  0.0000013,
     &  0.0000019,  0.0000026,  0.0000037,  0.0000052,  0.0000073,
     &  0.0000103,  0.0000145,  0.0000204,  0.0000285,  0.0000398,
     &  0.0000553,  0.0000764,  0.0001047,  0.0001423,  0.0001914,
     &  0.0002545,  0.0003344,  0.0004345,  0.0005589,  0.0007122,
     &  0.0009003,  0.0011304,  0.0014112,  0.0017531,  0.0021673,
     &  0.0026656,  0.0032598,  0.0039609,  0.0047786,  0.0057211,
     &  0.0067954,  0.0080078,  0.0093646/
      data ((cah(i,j),i=1,43),j= 17, 17)/
     &  0.0000001,  0.0000001,  0.0000001,  0.0000002,  0.0000002,
     &  0.0000003,  0.0000005,  0.0000007,  0.0000009,  0.0000013,
     &  0.0000018,  0.0000026,  0.0000037,  0.0000052,  0.0000073,
     &  0.0000103,  0.0000145,  0.0000203,  0.0000284,  0.0000397,
     &  0.0000551,  0.0000761,  0.0001044,  0.0001419,  0.0001908,
     &  0.0002536,  0.0003332,  0.0004330,  0.0005568,  0.0007095,
     &  0.0008968,  0.0011259,  0.0014054,  0.0017458,  0.0021123,
     &  0.0026542,  0.0032457,  0.0039435,  0.0047573,  0.0056951,
     &  0.0067640,  0.0079700,  0.0093194/
      data ((cah(i,j),i=1,43),j= 18, 18)/
     &  0.0000001,  0.0000001,  0.0000001,  0.0000002,  0.0000002,
     &  0.0000003,  0.0000005,  0.0000007,  0.0000009,  0.0000013,
     &  0.0000018,  0.0000026,  0.0000037,  0.0000052,  0.0000073,
     &  0.0000102,  0.0000144,  0.0000202,  0.0000283,  0.0000395,
     &  0.0000549,  0.0000758,  0.0001040,  0.0001413,  0.0001900,
     &  0.0002525,  0.0003318,  0.0004311,  0.0005543,  0.0007063,
     &  0.0008926,  0.0011204,  0.0013985,  0.0017370,  0.0021470,
     &  0.0026404,  0.0032285,  0.0039224,  0.0047315,  0.0056637,
     &  0.0067260,  0.0079245,  0.0092651/
      data ((cah(i,j),i=1,43),j= 19, 19)/
     &  0.0000001,  0.0000001,  0.0000001,  0.0000002,  0.0000002,
     &  0.0000003,  0.0000005,  0.0000006,  0.0000009,  0.0000013,
     &  0.0000018,  0.0000026,  0.0000036,  0.0000051,  0.0000072,
     &  0.0000102,  0.0000143,  0.0000201,  0.0000282,  0.0000393,
     &  0.0000546,  0.0000754,  0.0001034,  0.0001406,  0.0001890,
     &  0.0002512,  0.0003300,  0.0004287,  0.0005513,  0.0007023,
     &  0.0008875,  0.0011139,  0.0013901,  0.0017264,  0.0021337,
     &  0.0026238,  0.0032080,  0.0038971,  0.0047005,  0.0056261,
     &  0.0066806,  0.0078701,  0.0092003/
      data ((cah(i,j),i=1,43),j= 20, 20)/
     &  0.0000001,  0.0000001,  0.0000001,  0.0000002,  0.0000002,
     &  0.0000003,  0.0000005,  0.0000006,  0.0000009,  0.0000013,
     &  0.0000018,  0.0000026,  0.0000036,  0.0000051,  0.0000072,
     &  0.0000101,  0.0000142,  0.0000200,  0.0000280,  0.0000391,
     &  0.0000543,  0.0000750,  0.0001028,  0.0001397,  0.0001878,
     &  0.0002496,  0.0003279,  0.0004259,  0.0005476,  0.0006975,
     &  0.0008813,  0.0011060,  0.0013802,  0.0017138,  0.0021179,
     &  0.0026040,  0.0031835,  0.0038670,  0.0046637,  0.0055814,
     &  0.0066267,  0.0078055,  0.0091235/
      data ((cah(i,j),i=1,43),j= 21, 21)/
     &  0.0000001,  0.0000001,  0.0000001,  0.0000002,  0.0000002,
     &  0.0000003,  0.0000005,  0.0000006,  0.0000009,  0.0000013,
     &  0.0000018,  0.0000025,  0.0000036,  0.0000051,  0.0000071,
     &  0.0000100,  0.0000141,  0.0000198,  0.0000278,  0.0000388,
     &  0.0000539,  0.0000744,  0.0001020,  0.0001386,  0.0001863,
     &  0.0002477,  0.0003253,  0.0004226,  0.0005432,  0.0006918,
     &  0.0008740,  0.0010966,  0.0013683,  0.0016988,  0.0020991,
     &  0.0025806,  0.0031545,  0.0038313,  0.0046201,  0.0055285,
     &  0.0065630,  0.0077294,  0.0090332/
      data ((cah(i,j),i=1,43),j= 22, 22)/
     &  0.0000001,  0.0000001,  0.0000001,  0.0000002,  0.0000002,
     &  0.0000003,  0.0000004,  0.0000006,  0.0000009,  0.0000013,
     &  0.0000018,  0.0000025,  0.0000036,  0.0000050,  0.0000071,
     &  0.0000100,  0.0000140,  0.0000197,  0.0000275,  0.0000384,
     &  0.0000534,  0.0000737,  0.0001011,  0.0001373,  0.0001846,
     &  0.0002453,  0.0003222,  0.0004185,  0.0005265,  0.0006850,
     &  0.0008652,  0.0010855,  0.0013541,  0.0016809,  0.0020768,
     &  0.0025528,  0.0031202,  0.0037892,  0.0045688,  0.0054664,
     &  0.0064883,  0.0076402,  0.0089277/
      data ((cah(i,j),i=1,43),j= 23, 23)/
     &  0.0000001,  0.0000001,  0.0000001,  0.0000002,  0.0000002,
     &  0.0000003,  0.0000004,  0.0000006,  0.0000009,  0.0000013,
     &  0.0000018,  0.0000025,  0.0000035,  0.0000050,  0.0000070,
     &  0.0000098,  0.0000138,  0.0000194,  0.0000272,  0.0000380,
     &  0.0000528,  0.0000729,  0.0000999,  0.0001357,  0.0001825,
     &  0.0002425,  0.0003185,  0.0004137,  0.0005316,  0.0006769,
     &  0.0008548,  0.0010722,  0.0013373,  0.0016599,  0.0020504,
     &  0.0025201,  0.0030799,  0.0037398,  0.0045087,  0.0053938,
     &  0.0064013,  0.0075366,  0.0088053/
      data ((cah(i,j),i=1,43),j= 24, 24)/
     &  0.0000001,  0.0000001,  0.0000001,  0.0000002,  0.0000002,
     &  0.0000003,  0.0000004,  0.0000006,  0.0000009,  0.0000012,
     &  0.0000017,  0.0000025,  0.0000035,  0.0000049,  0.0000069,
     &  0.0000097,  0.0000137,  0.0000192,  0.0000268,  0.0000375,
     &  0.0000520,  0.0000719,  0.0000986,  0.0001339,  0.0001800,
     &  0.0002392,  0.0003142,  0.0004079,  0.0005242,  0.0006673,
     &  0.0008426,  0.0010567,  0.0013177,  0.0016352,  0.0020196,
     &  0.0024820,  0.0030330,  0.0036825,  0.0044391,  0.0053098,
     &  0.0063007,  0.0074172,  0.0084815/
      data ((cah(i,j),i=1,43),j= 25, 25)/
     &  0.0000001,  0.0000001,  0.0000001,  0.0000002,  0.0000002,
     &  0.0000003,  0.0000004,  0.0000006,  0.0000009,  0.0000012,
     &  0.0000017,  0.0000024,  0.0000034,  0.0000048,  0.0000068,
     &  0.0000096,  0.0000134,  0.0000189,  0.0000264,  0.0000369,
     &  0.0000512,  0.0000708,  0.0000970,  0.0001318,  0.0001772,
     &  0.0002354,  0.0003091,  0.0004013,  0.0005156,  0.0006562,
     &  0.0008284,  0.0010386,  0.0012949,  0.0016066,  0.0019840,
     &  0.0024379,  0.0029788,  0.0036164,  0.0043590,  0.0052135,
     &  0.0061857,  0.0072808,  0.0085042/
      data ((cah(i,j),i=1,43),j= 26, 26)/
     &  0.0000001,  0.0000001,  0.0000001,  0.0000002,  0.0000002,
     &  0.0000003,  0.0000004,  0.0000006,  0.0000008,  0.0000012,
     &  0.0000017,  0.0000024,  0.0000034,  0.0000047,  0.0000067,
     &  0.0000094,  0.0000132,  0.0000185,  0.0000259,  0.0000362,
     &  0.0000503,  0.0000695,  0.0000952,  0.0001294,  0.0001739,
     &  0.0002310,  0.0003033,  0.0003937,  0.0005057,  0.0006435,
     &  0.0008121,  0.0010180,  0.0012688,  0.0015739,  0.0019434,
     &  0.0023877,  0.0029172,  0.0035413,  0.0042681,  0.0051043,
     &  0.0060554,  0.0071267,  0.0083234/
      data ((cah(i,j),i=1,43),j= 27, 27)/
     &  0.0000001,  0.0000001,  0.0000001,  0.0000001,  0.0000002,
     &  0.0000003,  0.0000004,  0.0000006,  0.0000008,  0.0000012,
     &  0.0000016,  0.0000023,  0.0000033,  0.0000046,  0.0000065,
     &  0.0000092,  0.0000129,  0.0000181,  0.0000254,  0.0000355,
     &  0.0000493,  0.0000680,  0.0000933,  0.0001267,  0.0001702,
     &  0.0002261,  0.0002968,  0.0003852,  0.0004946,  0.0006291,
     &  0.0007937,  0.0009946,  0.0012394,  0.0015370,  0.0018975,
     &  0.0023310,  0.0028478,  0.0034568,  0.0041660,  0.0049818,
     &  0.0059096,  0.0069544,  0.0081215/
      data ((cah(i,j),i=1,43),j= 28, 28)/
     &  0.0000001,  0.0000001,  0.0000001,  0.0000001,  0.0000002,
     &  0.0000003,  0.0000004,  0.0000006,  0.0000008,  0.0000011,
     &  0.0000016,  0.0000023,  0.0000032,  0.0000045,  0.0000064,
     &  0.0000090,  0.0000126,  0.0000177,  0.0000248,  0.0000346,
     &  0.0000481,  0.0000664,  0.0000910,  0.0001236,  0.0001661,
     &  0.0002206,  0.0002895,  0.0003755,  0.0004821,  0.0006130,
     &  0.0007731,  0.0009685,  0.0012065,  0.0014959,  0.0018463,
     &  0.0022680,  0.0027705,  0.0033629,  0.0040526,  0.0048459,
     &  0.0057480,  0.0067639,  0.0078987/
      data ((cah(i,j),i=1,43),j= 29, 29)/
     &  0.0000000,  0.0000001,  0.0000001,  0.0000001,  0.0000002,
     &  0.0000003,  0.0000004,  0.0000006,  0.0000008,  0.0000011,
     &  0.0000016,  0.0000022,  0.0000031,  0.0000044,  0.0000062,
     &  0.0000087,  0.0000123,  0.0000173,  0.0000242,  0.0000330,
     &  0.0000468,  0.0000646,  0.0000886,  0.0001203,  0.0001616,
     &  0.0002145,  0.0002814,  0.0003649,  0.0004682,  0.0005951,
     &  0.0007503,  0.0009396,  0.0011701,  0.0014505,  0.0017900,
     &  0.0021986,  0.0026857,  0.0032598,  0.0039283,  0.0046971,
     &  0.0055713,  0.0065558,  0.0076557/
      data ((cah(i,j),i=1,43),j= 30, 30)/
     &  0.0000000,  0.0000001,  0.0000001,  0.0000001,  0.0000002,
     &  0.0000003,  0.0000004,  0.0000005,  0.0000008,  0.0000011,
     &  0.0000015,  0.0000021,  0.0000030,  0.0000043,  0.0000060,
     &  0.0000085,  0.0000119,  0.0000167,  0.0000234,  0.0000327,
     &  0.0000454,  0.0000627,  0.0000859,  0.0001166,  0.0001566,
     &  0.0002078,  0.0002724,  0.0003531,  0.0004529,  0.0005755,
     &  0.0007253,  0.0009079,  0.0011304,  0.0014010,  0.0017287,
     &  0.0021232,  0.0025935,  0.0031480,  0.0037936,  0.0045361,
     &  0.0053805,  0.0063314,  0.0073941/
      data ((cah(i,j),i=1,43),j= 31, 31)/
     &  0.0000000,  0.0000001,  0.0000001,  0.0000001,  0.0000002,
     &  0.0000003,  0.0000004,  0.0000005,  0.0000007,  0.0000010,
     &  0.0000015,  0.0000021,  0.0000029,  0.0000041,  0.0000058,
     &  0.0000082,  0.0000115,  0.0000162,  0.0000226,  0.0000316,
     &  0.0000438,  0.0000605,  0.0000829,  0.0001125,  0.0001510,
     &  0.0002004,  0.0002626,  0.0003402,  0.0004362,  0.0005540,
     &  0.0006980,  0.0008736,  0.0010874,  0.0013476,  0.0016627,
     &  0.0020421,  0.0024947,  0.0030283,  0.0036497,  0.0043644,
     &  0.0051772,  0.0060928,  0.0071164/
      data ((cah(i,j),i=1,43),j= 32, 32)/
     &  0.0000000,  0.0000001,  0.0000001,  0.0000001,  0.0000002,
     &  0.0000003,  0.0000004,  0.0000005,  0.0000007,  0.0000010,
     &  0.0000014,  0.0000020,  0.0000028,  0.0000040,  0.0000056,
     &  0.0000079,  0.0000111,  0.0000155,  0.0000218,  0.0000303,
     &  0.0000421,  0.0000582,  0.0000797,  0.0001081,  0.0001450,
     &  0.0001923,  0.0002519,  0.0003262,  0.0004180,  0.0005308,
     &  0.0006686,  0.0008367,  0.0010414,  0.0012905,  0.0015925,
     &  0.0019561,  0.0023900,  0.0029017,  0.0034978,  0.0041836,
     &  0.0049638,  0.0058430,  0.0068264/
      data ((cah(i,j),i=1,43),j= 33, 33)/
     &  0.0000000,  0.0000001,  0.0000001,  0.0000001,  0.0000002,
     &  0.0000002,  0.0000003,  0.0000005,  0.0000007,  0.0000010,
     &  0.0000014,  0.0000019,  0.0000027,  0.0000038,  0.0000053,
     &  0.0000075,  0.0000106,  0.0000149,  0.0000208,  0.0000290,
     &  0.0000403,  0.0000556,  0.0000761,  0.0001032,  0.0001384,
     &  0.0001834,  0.0002402,  0.0003110,  0.0003985,  0.0005059,
     &  0.0006372,  0.0007974,  0.0009926,  0.0012302,  0.0015185,
     &  0.0018657,  0.0022803,  0.0027696,  0.0033398,  0.0039960,
     &  0.0047430,  0.0055851,  0.0065278/
      data ((cah(i,j),i=1,43),j= 34, 34)/
     &  0.0000000,  0.0000001,  0.0000001,  0.0000001,  0.0000002,
     &  0.0000002,  0.0000003,  0.0000005,  0.0000006,  0.0000009,
     &  0.0000013,  0.0000018,  0.0000026,  0.0000036,  0.0000051,
     &  0.0000071,  0.0000100,  0.0000141,  0.0000197,  0.0000275,
     &  0.0000382,  0.0000527,  0.0000722,  0.0000979,  0.0001312,
     &  0.0001739,  0.0002277,  0.0002947,  0.0003775,  0.0004793,
     &  0.0006038,  0.0007558,  0.0009412,  0.0011671,  0.0014412,
     &  0.0017717,  0.0021208,  0.0026329,  0.0031768,  0.0038033,
     &  0.0045168,  0.0053220,  0.0062240/
      data ((cah(i,j),i=1,43),j= 35, 35)/
     &  0.0000000,  0.0000001,  0.0000001,  0.0000001,  0.0000002,
     &  0.0000002,  0.0000003,  0.0000004,  0.0000006,  0.0000009,
     &  0.0000012,  0.0000017,  0.0000024,  0.0000034,  0.0000048,
     &  0.0000067,  0.0000095,  0.0000133,  0.0000186,  0.0000259,
     &  0.0000360,  0.0000496,  0.0000679,  0.0000921,  0.0001235,
     &  0.0001637,  0.0002143,  0.0002773,  0.0003554,  0.0004513,
     &  0.0005688,  0.0007124,  0.0008876,  0.0011014,  0.0013610,
     &  0.0016745,  0.0020493,  0.0024925,  0.0030099,  0.0036066,
     &  0.0042868,  0.0050553,  0.0059171/
      data ((cah(i,j),i=1,43),j= 36, 36)/
     &  0.0000000,  0.0000001,  0.0000001,  0.0000001,  0.0000001,
     &  0.0000002,  0.0000003,  0.0000004,  0.0000006,  0.0000008,
     &  0.0000011,  0.0000016,  0.0000022,  0.0000032,  0.0000045,
     &  0.0000063,  0.0000088,  0.0000124,  0.0000173,  0.0000242,
     &  0.0000336,  0.0000463,  0.0000634,  0.0000860,  0.0001153,
     &  0.0001528,  0.0002001,  0.0002591,  0.0003322,  0.0004221,
     &  0.0005323,  0.0006672,  0.0008322,  0.0010335,  0.0012785,
     &  0.0015746,  0.0019293,  0.0023491,  0.0028399,  0.0034067,
     &  0.0040539,  0.0047860,  0.0056083/
      data ((cah(i,j),i=1,43),j= 37, 37)/
     &  0.0000000,  0.0000000,  0.0000001,  0.0000001,  0.0000001,
     &  0.0000002,  0.0000003,  0.0000004,  0.0000005,  0.0000007,
     &  0.0000010,  0.0000015,  0.0000021,  0.0000029,  0.0000041,
     &  0.0000058,  0.0000082,  0.0000114,  0.0000160,  0.0000223,
     &  0.0000310,  0.0000428,  0.0000586,  0.0000795,  0.0001067,
     &  0.0001414,  0.0001853,  0.0002401,  0.0003081,  0.0003918,
     &  0.0004947,  0.0006208,  0.0007751,  0.0009639,  0.0011940,
     &  0.0014726,  0.0018069,  0.0022032,  0.0026674,  0.0032043,
     &  0.0038186,  0.0045147,  0.0052979/


c-----coa is the co2 absorptance in strong absorption regions of band 11

!!       include "coa.data"
      data ((coa(i,j),i=1,62),j=  1,  1)/
     &  0.0000080,  0.0000089,  0.0000098,  0.0000106,  0.0000114,
     &  0.0000121,  0.0000128,  0.0000134,  0.0000140,  0.0000146,
     &  0.0000152,  0.0000158,  0.0000163,  0.0000168,  0.0000173,
     &  0.0000178,  0.0000182,  0.0000186,  0.0000191,  0.0000195,
     &  0.0000199,  0.0000202,  0.0000206,  0.0000210,  0.0000213,
     &  0.0000217,  0.0000220,  0.0000223,  0.0000226,  0.0000229,
     &  0.0000232,  0.0000235,  0.0000238,  0.0000241,  0.0000244,
     &  0.0000246,  0.0000249,  0.0000252,  0.0000254,  0.0000257,
     &  0.0000259,  0.0000261,  0.0000264,  0.0000266,  0.0000268,
     &  0.0000271,  0.0000273,  0.0000275,  0.0000277,  0.0000279,
     &  0.0000281,  0.0000283,  0.0000285,  0.0000287,  0.0000289,
     &  0.0000291,  0.0000293,  0.0000295,  0.0000297,  0.0000298,
     &  0.0000300,  0.0000302/
      data ((coa(i,j),i=1,62),j=  2,  2)/
     &  0.0000085,  0.0000095,  0.0000104,  0.0000113,  0.0000121,
     &  0.0000128,  0.0000136,  0.0000143,  0.0000149,  0.0000155,
     &  0.0000161,  0.0000167,  0.0000172,  0.0000178,  0.0000183,
     &  0.0000187,  0.0000192,  0.0000196,  0.0000201,  0.0000205,
     &  0.0000209,  0.0000213,  0.0000217,  0.0000220,  0.0000224,
     &  0.0000227,  0.0000231,  0.0000234,  0.0000237,  0.0000240,
     &  0.0000243,  0.0000246,  0.0000249,  0.0000252,  0.0000255,
     &  0.0000258,  0.0000260,  0.0000263,  0.0000266,  0.0000268,
     &  0.0000271,  0.0000273,  0.0000275,  0.0000278,  0.0000280,
     &  0.0000282,  0.0000285,  0.0000287,  0.0000289,  0.0000291,
     &  0.0000293,  0.0000295,  0.0000297,  0.0000299,  0.0000301,
     &  0.0000303,  0.0000305,  0.0000307,  0.0000309,  0.0000311,
     &  0.0000313,  0.0000314/
      data ((coa(i,j),i=1,62),j=  3,  3)/
     &  0.0000095,  0.0000106,  0.0000116,  0.0000125,  0.0000134,
     &  0.0000143,  0.0000150,  0.0000158,  0.0000165,  0.0000171,
     &  0.0000178,  0.0000184,  0.0000189,  0.0000195,  0.0000200,
     &  0.0000205,  0.0000210,  0.0000215,  0.0000219,  0.0000223,
     &  0.0000228,  0.0000232,  0.0000235,  0.0000239,  0.0000243,
     &  0.0000247,  0.0000250,  0.0000253,  0.0000257,  0.0000260,
     &  0.0000263,  0.0000266,  0.0000269,  0.0000272,  0.0000275,
     &  0.0000278,  0.0000281,  0.0000283,  0.0000286,  0.0000289,
     &  0.0000291,  0.0000294,  0.0000296,  0.0000299,  0.0000301,
     &  0.0000303,  0.0000306,  0.0000308,  0.0000310,  0.0000312,
     &  0.0000315,  0.0000317,  0.0000319,  0.0000321,  0.0000323,
     &  0.0000325,  0.0000327,  0.0000329,  0.0000331,  0.0000333,
     &  0.0000335,  0.0000329/
      data ((coa(i,j),i=1,62),j=  4,  4)/
     &  0.0000100,  0.0000111,  0.0000122,  0.0000131,  0.0000141,
     &  0.0000149,  0.0000157,  0.0000165,  0.0000172,  0.0000179,
     &  0.0000185,  0.0000191,  0.0000197,  0.0000203,  0.0000208,
     &  0.0000213,  0.0000218,  0.0000223,  0.0000227,  0.0000232,
     &  0.0000236,  0.0000240,  0.0000244,  0.0000248,  0.0000252,
     &  0.0000255,  0.0000259,  0.0000262,  0.0000266,  0.0000269,
     &  0.0000272,  0.0000275,  0.0000278,  0.0000281,  0.0000284,
     &  0.0000287,  0.0000290,  0.0000293,  0.0000295,  0.0000298,
     &  0.0000300,  0.0000303,  0.0000306,  0.0000308,  0.0000310,
     &  0.0000313,  0.0000315,  0.0000317,  0.0000320,  0.0000322,
     &  0.0000324,  0.0000326,  0.0000328,  0.0000331,  0.0000333,
     &  0.0000335,  0.0000330,  0.0000339,  0.0000341,  0.0000343,
     &  0.0000345,  0.0000346/
      data ((coa(i,j),i=1,62),j=  5,  5)/
     &  0.0000109,  0.0000121,  0.0000132,  0.0000143,  0.0000152,
     &  0.0000161,  0.0000170,  0.0000178,  0.0000185,  0.0000192,
     &  0.0000199,  0.0000205,  0.0000211,  0.0000217,  0.0000222,
     &  0.0000228,  0.0000233,  0.0000238,  0.0000242,  0.0000247,
     &  0.0000251,  0.0000255,  0.0000259,  0.0000263,  0.0000267,
     &  0.0000271,  0.0000275,  0.0000278,  0.0000282,  0.0000285,
     &  0.0000288,  0.0000291,  0.0000295,  0.0000298,  0.0000301,
     &  0.0000304,  0.0000307,  0.0000309,  0.0000312,  0.0000315,
     &  0.0000318,  0.0000320,  0.0000323,  0.0000325,  0.0000328,
     &  0.0000330,  0.0000333,  0.0000335,  0.0000330,  0.0000340,
     &  0.0000342,  0.0000344,  0.0000346,  0.0000348,  0.0000351,
     &  0.0000353,  0.0000355,  0.0000357,  0.0000359,  0.0000361,
     &  0.0000363,  0.0000365/
      data ((coa(i,j),i=1,62),j=  6,  6)/
     &  0.0000117,  0.0000130,  0.0000142,  0.0000153,  0.0000163,
     &  0.0000173,  0.0000181,  0.0000190,  0.0000197,  0.0000204,
     &  0.0000211,  0.0000218,  0.0000224,  0.0000230,  0.0000235,
     &  0.0000241,  0.0000246,  0.0000251,  0.0000256,  0.0000260,
     &  0.0000265,  0.0000269,  0.0000273,  0.0000277,  0.0000281,
     &  0.0000285,  0.0000289,  0.0000293,  0.0000296,  0.0000299,
     &  0.0000303,  0.0000306,  0.0000309,  0.0000313,  0.0000316,
     &  0.0000319,  0.0000322,  0.0000324,  0.0000327,  0.0000330,
     &  0.0000333,  0.0000336,  0.0000331,  0.0000341,  0.0000343,
     &  0.0000346,  0.0000348,  0.0000351,  0.0000353,  0.0000355,
     &  0.0000358,  0.0000360,  0.0000362,  0.0000365,  0.0000367,
     &  0.0000369,  0.0000371,  0.0000373,  0.0000375,  0.0000377,
     &  0.0000379,  0.0000381/
      data ((coa(i,j),i=1,62),j=  7,  7)/
     &  0.0000125,  0.0000139,  0.0000151,  0.0000163,  0.0000173,
     &  0.0000183,  0.0000192,  0.0000200,  0.0000208,  0.0000216,
     &  0.0000223,  0.0000229,  0.0000236,  0.0000242,  0.0000247,
     &  0.0000253,  0.0000258,  0.0000263,  0.0000268,  0.0000273,
     &  0.0000277,  0.0000282,  0.0000286,  0.0000290,  0.0000294,
     &  0.0000298,  0.0000302,  0.0000306,  0.0000309,  0.0000313,
     &  0.0000316,  0.0000320,  0.0000323,  0.0000326,  0.0000329,
     &  0.0000332,  0.0000335,  0.0000331,  0.0000341,  0.0000344,
     &  0.0000347,  0.0000350,  0.0000352,  0.0000355,  0.0000358,
     &  0.0000360,  0.0000363,  0.0000365,  0.0000368,  0.0000370,
     &  0.0000372,  0.0000375,  0.0000377,  0.0000379,  0.0000382,
     &  0.0000384,  0.0000386,  0.0000388,  0.0000390,  0.0000392,
     &  0.0000394,  0.0000396/
      data ((coa(i,j),i=1,62),j=  8,  8)/
     &  0.0000132,  0.0000147,  0.0000160,  0.0000172,  0.0000183,
     &  0.0000193,  0.0000202,  0.0000210,  0.0000218,  0.0000226,
     &  0.0000233,  0.0000240,  0.0000246,  0.0000252,  0.0000258,
     &  0.0000264,  0.0000269,  0.0000274,  0.0000279,  0.0000284,
     &  0.0000289,  0.0000293,  0.0000298,  0.0000302,  0.0000306,
     &  0.0000310,  0.0000314,  0.0000318,  0.0000321,  0.0000325,
     &  0.0000328,  0.0000332,  0.0000335,  0.0000331,  0.0000342,
     &  0.0000345,  0.0000348,  0.0000351,  0.0000354,  0.0000357,
     &  0.0000360,  0.0000363,  0.0000365,  0.0000368,  0.0000371,
     &  0.0000373,  0.0000376,  0.0000378,  0.0000381,  0.0000383,
     &  0.0000386,  0.0000388,  0.0000391,  0.0000393,  0.0000395,
     &  0.0000397,  0.0000400,  0.0000402,  0.0000404,  0.0000406,
     &  0.0000408,  0.0000411/
      data ((coa(i,j),i=1,62),j=  9,  9)/
     &  0.0000143,  0.0000158,  0.0000172,  0.0000184,  0.0000195,
     &  0.0000206,  0.0000215,  0.0000224,  0.0000232,  0.0000240,
     &  0.0000247,  0.0000254,  0.0000261,  0.0000267,  0.0000273,
     &  0.0000279,  0.0000284,  0.0000290,  0.0000295,  0.0000300,
     &  0.0000305,  0.0000309,  0.0000314,  0.0000318,  0.0000322,
     &  0.0000326,  0.0000330,  0.0000334,  0.0000331,  0.0000342,
     &  0.0000345,  0.0000349,  0.0000352,  0.0000356,  0.0000359,
     &  0.0000362,  0.0000365,  0.0000368,  0.0000371,  0.0000374,
     &  0.0000377,  0.0000380,  0.0000383,  0.0000386,  0.0000389,
     &  0.0000391,  0.0000394,  0.0000397,  0.0000399,  0.0000402,
     &  0.0000404,  0.0000407,  0.0000409,  0.0000412,  0.0000414,
     &  0.0000416,  0.0000419,  0.0000421,  0.0000423,  0.0000426,
     &  0.0000428,  0.0000430/
      data ((coa(i,j),i=1,62),j= 10, 10)/
     &  0.0000153,  0.0000169,  0.0000183,  0.0000196,  0.0000207,
     &  0.0000218,  0.0000227,  0.0000236,  0.0000245,  0.0000253,
     &  0.0000260,  0.0000267,  0.0000274,  0.0000281,  0.0000287,
     &  0.0000293,  0.0000298,  0.0000304,  0.0000309,  0.0000314,
     &  0.0000319,  0.0000324,  0.0000328,  0.0000333,  0.0000330,
     &  0.0000341,  0.0000345,  0.0000349,  0.0000353,  0.0000357,
     &  0.0000361,  0.0000364,  0.0000368,  0.0000371,  0.0000375,
     &  0.0000378,  0.0000381,  0.0000384,  0.0000387,  0.0000391,
     &  0.0000394,  0.0000397,  0.0000399,  0.0000402,  0.0000405,
     &  0.0000408,  0.0000411,  0.0000413,  0.0000416,  0.0000419,
     &  0.0000421,  0.0000424,  0.0000426,  0.0000429,  0.0000431,
     &  0.0000434,  0.0000436,  0.0000439,  0.0000441,  0.0000443,
     &  0.0000446,  0.0000448/
      data ((coa(i,j),i=1,62),j= 11, 11)/
     &  0.0000165,  0.0000182,  0.0000196,  0.0000209,  0.0000221,
     &  0.0000232,  0.0000242,  0.0000251,  0.0000260,  0.0000268,
     &  0.0000276,  0.0000283,  0.0000290,  0.0000297,  0.0000303,
     &  0.0000309,  0.0000315,  0.0000321,  0.0000326,  0.0000331,
     &  0.0000336,  0.0000341,  0.0000346,  0.0000350,  0.0000355,
     &  0.0000359,  0.0000363,  0.0000367,  0.0000371,  0.0000375,
     &  0.0000379,  0.0000383,  0.0000386,  0.0000390,  0.0000394,
     &  0.0000397,  0.0000400,  0.0000404,  0.0000407,  0.0000410,
     &  0.0000413,  0.0000416,  0.0000419,  0.0000422,  0.0000425,
     &  0.0000428,  0.0000431,  0.0000434,  0.0000437,  0.0000439,
     &  0.0000442,  0.0000445,  0.0000447,  0.0000450,  0.0000453,
     &  0.0000455,  0.0000458,  0.0000460,  0.0000463,  0.0000465,
     &  0.0000468,  0.0000470/
      data ((coa(i,j),i=1,62),j= 12, 12)/
     &  0.0000173,  0.0000190,  0.0000205,  0.0000219,  0.0000231,
     &  0.0000242,  0.0000252,  0.0000262,  0.0000271,  0.0000279,
     &  0.0000287,  0.0000294,  0.0000301,  0.0000308,  0.0000314,
     &  0.0000320,  0.0000326,  0.0000332,  0.0000330,  0.0000343,
     &  0.0000348,  0.0000353,  0.0000358,  0.0000362,  0.0000367,
     &  0.0000371,  0.0000376,  0.0000380,  0.0000384,  0.0000388,
     &  0.0000392,  0.0000396,  0.0000399,  0.0000403,  0.0000407,
     &  0.0000410,  0.0000414,  0.0000417,  0.0000420,  0.0000424,
     &  0.0000427,  0.0000430,  0.0000433,  0.0000436,  0.0000439,
     &  0.0000442,  0.0000445,  0.0000448,  0.0000451,  0.0000454,
     &  0.0000457,  0.0000459,  0.0000462,  0.0000465,  0.0000468,
     &  0.0000470,  0.0000473,  0.0000475,  0.0000478,  0.0000481,
     &  0.0000483,  0.0000486/
      data ((coa(i,j),i=1,62),j= 13, 13)/
     &  0.0000186,  0.0000204,  0.0000219,  0.0000233,  0.0000246,
     &  0.0000257,  0.0000268,  0.0000277,  0.0000286,  0.0000295,
     &  0.0000303,  0.0000311,  0.0000318,  0.0000325,  0.0000331,
     &  0.0000331,  0.0000344,  0.0000350,  0.0000355,  0.0000361,
     &  0.0000366,  0.0000371,  0.0000376,  0.0000381,  0.0000386,
     &  0.0000390,  0.0000395,  0.0000399,  0.0000403,  0.0000407,
     &  0.0000412,  0.0000416,  0.0000419,  0.0000423,  0.0000427,
     &  0.0000431,  0.0000434,  0.0000438,  0.0000441,  0.0000445,
     &  0.0000448,  0.0000451,  0.0000455,  0.0000458,  0.0000461,
     &  0.0000464,  0.0000467,  0.0000470,  0.0000473,  0.0000476,
     &  0.0000479,  0.0000482,  0.0000485,  0.0000488,  0.0000491,
     &  0.0000494,  0.0000497,  0.0000499,  0.0000502,  0.0000505,
     &  0.0000507,  0.0000510/
      data ((coa(i,j),i=1,62),j= 14, 14)/
     &  0.0000198,  0.0000216,  0.0000232,  0.0000246,  0.0000259,
     &  0.0000271,  0.0000281,  0.0000291,  0.0000301,  0.0000310,
     &  0.0000318,  0.0000326,  0.0000333,  0.0000340,  0.0000347,
     &  0.0000354,  0.0000360,  0.0000366,  0.0000372,  0.0000377,
     &  0.0000383,  0.0000388,  0.0000393,  0.0000398,  0.0000403,
     &  0.0000408,  0.0000412,  0.0000417,  0.0000421,  0.0000425,
     &  0.0000430,  0.0000434,  0.0000438,  0.0000442,  0.0000446,
     &  0.0000449,  0.0000453,  0.0000457,  0.0000461,  0.0000464,
     &  0.0000468,  0.0000471,  0.0000475,  0.0000478,  0.0000481,
     &  0.0000485,  0.0000488,  0.0000491,  0.0000494,  0.0000498,
     &  0.0000501,  0.0000504,  0.0000507,  0.0000510,  0.0000513,
     &  0.0000516,  0.0000519,  0.0000522,  0.0000524,  0.0000527,
     &  0.0000530,  0.0000533/
      data ((coa(i,j),i=1,62),j= 15, 15)/
     &  0.0000209,  0.0000228,  0.0000244,  0.0000258,  0.0000271,
     &  0.0000283,  0.0000294,  0.0000305,  0.0000314,  0.0000323,
     &  0.0000332,  0.0000340,  0.0000347,  0.0000354,  0.0000361,
     &  0.0000368,  0.0000375,  0.0000381,  0.0000387,  0.0000392,
     &  0.0000398,  0.0000404,  0.0000409,  0.0000414,  0.0000419,
     &  0.0000424,  0.0000429,  0.0000433,  0.0000438,  0.0000442,
     &  0.0000447,  0.0000451,  0.0000455,  0.0000459,  0.0000463,
     &  0.0000467,  0.0000471,  0.0000475,  0.0000479,  0.0000483,
     &  0.0000486,  0.0000490,  0.0000493,  0.0000497,  0.0000501,
     &  0.0000504,  0.0000507,  0.0000511,  0.0000514,  0.0000518,
     &  0.0000521,  0.0000524,  0.0000527,  0.0000530,  0.0000534,
     &  0.0000537,  0.0000540,  0.0000543,  0.0000546,  0.0000549,
     &  0.0000552,  0.0000555/
      data ((coa(i,j),i=1,62),j= 16, 16)/
     &  0.0000221,  0.0000240,  0.0000257,  0.0000272,  0.0000285,
     &  0.0000297,  0.0000308,  0.0000319,  0.0000329,  0.0000331,
     &  0.0000347,  0.0000355,  0.0000363,  0.0000370,  0.0000377,
     &  0.0000384,  0.0000391,  0.0000397,  0.0000404,  0.0000409,
     &  0.0000415,  0.0000421,  0.0000426,  0.0000432,  0.0000437,
     &  0.0000442,  0.0000447,  0.0000452,  0.0000456,  0.0000461,
     &  0.0000466,  0.0000470,  0.0000475,  0.0000479,  0.0000483,
     &  0.0000487,  0.0000491,  0.0000496,  0.0000500,  0.0000503,
     &  0.0000507,  0.0000511,  0.0000515,  0.0000519,  0.0000523,
     &  0.0000526,  0.0000530,  0.0000533,  0.0000537,  0.0000540,
     &  0.0000544,  0.0000547,  0.0000551,  0.0000554,  0.0000558,
     &  0.0000561,  0.0000564,  0.0000567,  0.0000571,  0.0000574,
     &  0.0000577,  0.0000580/
      data ((coa(i,j),i=1,62),j= 17, 17)/
     &  0.0000234,  0.0000254,  0.0000271,  0.0000286,  0.0000300,
     &  0.0000312,  0.0000324,  0.0000335,  0.0000345,  0.0000354,
     &  0.0000363,  0.0000372,  0.0000380,  0.0000387,  0.0000395,
     &  0.0000402,  0.0000409,  0.0000415,  0.0000422,  0.0000428,
     &  0.0000434,  0.0000440,  0.0000446,  0.0000451,  0.0000457,
     &  0.0000462,  0.0000467,  0.0000472,  0.0000477,  0.0000482,
     &  0.0000487,  0.0000492,  0.0000496,  0.0000501,  0.0000505,
     &  0.0000510,  0.0000514,  0.0000518,  0.0000523,  0.0000527,
     &  0.0000531,  0.0000535,  0.0000539,  0.0000543,  0.0000547,
     &  0.0000551,  0.0000555,  0.0000559,  0.0000562,  0.0000566,
     &  0.0000570,  0.0000573,  0.0000577,  0.0000581,  0.0000584,
     &  0.0000588,  0.0000591,  0.0000595,  0.0000598,  0.0000602,
     &  0.0000605,  0.0000608/
      data ((coa(i,j),i=1,62),j= 18, 18)/
     &  0.0000248,  0.0000268,  0.0000285,  0.0000301,  0.0000315,
     &  0.0000328,  0.0000340,  0.0000351,  0.0000362,  0.0000371,
     &  0.0000381,  0.0000389,  0.0000398,  0.0000406,  0.0000413,
     &  0.0000421,  0.0000428,  0.0000435,  0.0000442,  0.0000448,
     &  0.0000454,  0.0000460,  0.0000466,  0.0000472,  0.0000478,
     &  0.0000484,  0.0000489,  0.0000494,  0.0000500,  0.0000505,
     &  0.0000510,  0.0000515,  0.0000520,  0.0000525,  0.0000530,
     &  0.0000534,  0.0000539,  0.0000544,  0.0000548,  0.0000553,
     &  0.0000557,  0.0000561,  0.0000566,  0.0000570,  0.0000574,
     &  0.0000578,  0.0000582,  0.0000586,  0.0000590,  0.0000594,
     &  0.0000598,  0.0000602,  0.0000606,  0.0000610,  0.0000614,
     &  0.0000618,  0.0000621,  0.0000625,  0.0000629,  0.0000633,
     &  0.0000636,  0.0000640/
      data ((coa(i,j),i=1,62),j= 19, 19)/
     &  0.0000260,  0.0000281,  0.0000299,  0.0000315,  0.0000330,
     &  0.0000343,  0.0000355,  0.0000367,  0.0000377,  0.0000388,
     &  0.0000397,  0.0000406,  0.0000415,  0.0000423,  0.0000431,
     &  0.0000439,  0.0000446,  0.0000453,  0.0000460,  0.0000467,
     &  0.0000474,  0.0000480,  0.0000487,  0.0000493,  0.0000499,
     &  0.0000505,  0.0000510,  0.0000516,  0.0000522,  0.0000527,
     &  0.0000533,  0.0000538,  0.0000543,  0.0000548,  0.0000553,
     &  0.0000558,  0.0000563,  0.0000568,  0.0000573,  0.0000578,
     &  0.0000582,  0.0000587,  0.0000591,  0.0000596,  0.0000601,
     &  0.0000605,  0.0000609,  0.0000614,  0.0000618,  0.0000622,
     &  0.0000626,  0.0000631,  0.0000635,  0.0000639,  0.0000643,
     &  0.0000647,  0.0000651,  0.0000655,  0.0000659,  0.0000663,
     &  0.0000667,  0.0000670/
      data ((coa(i,j),i=1,62),j= 20, 20)/
     &  0.0000275,  0.0000296,  0.0000315,  0.0000332,  0.0000347,
     &  0.0000360,  0.0000373,  0.0000385,  0.0000396,  0.0000407,
     &  0.0000417,  0.0000426,  0.0000435,  0.0000444,  0.0000452,
     &  0.0000460,  0.0000468,  0.0000476,  0.0000483,  0.0000490,
     &  0.0000497,  0.0000504,  0.0000511,  0.0000517,  0.0000524,
     &  0.0000530,  0.0000536,  0.0000542,  0.0000548,  0.0000554,
     &  0.0000560,  0.0000566,  0.0000571,  0.0000577,  0.0000582,
     &  0.0000587,  0.0000593,  0.0000598,  0.0000603,  0.0000608,
     &  0.0000613,  0.0000618,  0.0000623,  0.0000628,  0.0000633,
     &  0.0000638,  0.0000642,  0.0000647,  0.0000652,  0.0000656,
     &  0.0000661,  0.0000665,  0.0000670,  0.0000674,  0.0000678,
     &  0.0000683,  0.0000687,  0.0000691,  0.0000695,  0.0000700,
     &  0.0000704,  0.0000708/
      data ((coa(i,j),i=1,62),j= 21, 21)/
     &  0.0000290,  0.0000312,  0.0000331,  0.0000349,  0.0000364,
     &  0.0000379,  0.0000392,  0.0000404,  0.0000416,  0.0000427,
     &  0.0000437,  0.0000447,  0.0000457,  0.0000466,  0.0000475,
     &  0.0000483,  0.0000492,  0.0000500,  0.0000507,  0.0000515,
     &  0.0000523,  0.0000530,  0.0000537,  0.0000544,  0.0000551,
     &  0.0000558,  0.0000564,  0.0000571,  0.0000577,  0.0000583,
     &  0.0000589,  0.0000596,  0.0000602,  0.0000607,  0.0000613,
     &  0.0000619,  0.0000625,  0.0000630,  0.0000636,  0.0000641,
     &  0.0000647,  0.0000652,  0.0000657,  0.0000663,  0.0000668,
     &  0.0000673,  0.0000678,  0.0000683,  0.0000688,  0.0000693,
     &  0.0000698,  0.0000702,  0.0000707,  0.0000712,  0.0000716,
     &  0.0000721,  0.0000726,  0.0000730,  0.0000735,  0.0000739,
     &  0.0000744,  0.0000748/
      data ((coa(i,j),i=1,62),j= 22, 22)/
     &  0.0000306,  0.0000329,  0.0000349,  0.0000366,  0.0000383,
     &  0.0000398,  0.0000411,  0.0000424,  0.0000436,  0.0000448,
     &  0.0000459,  0.0000469,  0.0000479,  0.0000489,  0.0000499,
     &  0.0000508,  0.0000516,  0.0000525,  0.0000533,  0.0000542,
     &  0.0000549,  0.0000557,  0.0000565,  0.0000572,  0.0000580,
     &  0.0000587,  0.0000594,  0.0000601,  0.0000608,  0.0000615,
     &  0.0000621,  0.0000628,  0.0000634,  0.0000640,  0.0000647,
     &  0.0000653,  0.0000659,  0.0000665,  0.0000671,  0.0000677,
     &  0.0000683,  0.0000688,  0.0000694,  0.0000700,  0.0000705,
     &  0.0000711,  0.0000716,  0.0000721,  0.0000727,  0.0000732,
     &  0.0000737,  0.0000742,  0.0000747,  0.0000752,  0.0000757,
     &  0.0000762,  0.0000767,  0.0000772,  0.0000777,  0.0000782,
     &  0.0000786,  0.0000791/
      data ((coa(i,j),i=1,62),j= 23, 23)/
     &  0.0000323,  0.0000347,  0.0000368,  0.0000386,  0.0000403,
     &  0.0000419,  0.0000433,  0.0000447,  0.0000459,  0.0000472,
     &  0.0000483,  0.0000494,  0.0000505,  0.0000516,  0.0000526,
     &  0.0000535,  0.0000545,  0.0000554,  0.0000563,  0.0000572,
     &  0.0000580,  0.0000589,  0.0000597,  0.0000605,  0.0000613,
     &  0.0000621,  0.0000628,  0.0000636,  0.0000643,  0.0000650,
     &  0.0000657,  0.0000664,  0.0000671,  0.0000678,  0.0000685,
     &  0.0000692,  0.0000698,  0.0000705,  0.0000711,  0.0000717,
     &  0.0000724,  0.0000730,  0.0000736,  0.0000742,  0.0000748,
     &  0.0000754,  0.0000760,  0.0000765,  0.0000771,  0.0000777,
     &  0.0000782,  0.0000788,  0.0000793,  0.0000799,  0.0000804,
     &  0.0000809,  0.0000815,  0.0000820,  0.0000825,  0.0000830,
     &  0.0000835,  0.0000840/
      data ((coa(i,j),i=1,62),j= 24, 24)/
     &  0.0000341,  0.0000365,  0.0000387,  0.0000406,  0.0000424,
     &  0.0000440,  0.0000456,  0.0000470,  0.0000483,  0.0000496,
     &  0.0000509,  0.0000521,  0.0000532,  0.0000543,  0.0000554,
     &  0.0000564,  0.0000574,  0.0000584,  0.0000594,  0.0000603,
     &  0.0000613,  0.0000622,  0.0000630,  0.0000639,  0.0000648,
     &  0.0000656,  0.0000664,  0.0000672,  0.0000680,  0.0000688,
     &  0.0000696,  0.0000703,  0.0000711,  0.0000718,  0.0000725,
     &  0.0000732,  0.0000739,  0.0000746,  0.0000753,  0.0000760,
     &  0.0000767,  0.0000773,  0.0000780,  0.0000786,  0.0000793,
     &  0.0000799,  0.0000805,  0.0000811,  0.0000817,  0.0000823,
     &  0.0000829,  0.0000835,  0.0000841,  0.0000847,  0.0000853,
     &  0.0000858,  0.0000864,  0.0000870,  0.0000875,  0.0000881,
     &  0.0000886,  0.0000892/
      data ((coa(i,j),i=1,62),j= 25, 25)/
     &  0.0000359,  0.0000385,  0.0000408,  0.0000428,  0.0000447,
     &  0.0000464,  0.0000480,  0.0000495,  0.0000510,  0.0000524,
     &  0.0000537,  0.0000550,  0.0000562,  0.0000574,  0.0000585,
     &  0.0000597,  0.0000608,  0.0000618,  0.0000629,  0.0000639,
     &  0.0000649,  0.0000658,  0.0000668,  0.0000677,  0.0000686,
     &  0.0000695,  0.0000704,  0.0000713,  0.0000721,  0.0000730,
     &  0.0000738,  0.0000746,  0.0000754,  0.0000762,  0.0000770,
     &  0.0000777,  0.0000785,  0.0000792,  0.0000800,  0.0000807,
     &  0.0000814,  0.0000821,  0.0000828,  0.0000835,  0.0000842,
     &  0.0000849,  0.0000856,  0.0000862,  0.0000869,  0.0000875,
     &  0.0000882,  0.0000888,  0.0000894,  0.0000900,  0.0000907,
     &  0.0000913,  0.0000919,  0.0000925,  0.0000931,  0.0000936,
     &  0.0000942,  0.0000948/
      data ((coa(i,j),i=1,62),j= 26, 26)/
     &  0.0000380,  0.0000407,  0.0000431,  0.0000453,  0.0000473,
     &  0.0000491,  0.0000508,  0.0000525,  0.0000540,  0.0000555,
     &  0.0000569,  0.0000583,  0.0000596,  0.0000609,  0.0000622,
     &  0.0000634,  0.0000646,  0.0000657,  0.0000668,  0.0000679,
     &  0.0000690,  0.0000700,  0.0000711,  0.0000721,  0.0000731,
     &  0.0000740,  0.0000750,  0.0000759,  0.0000769,  0.0000778,
     &  0.0000786,  0.0000795,  0.0000804,  0.0000812,  0.0000821,
     &  0.0000829,  0.0000837,  0.0000845,  0.0000853,  0.0000861,
     &  0.0000869,  0.0000876,  0.0000884,  0.0000891,  0.0000899,
     &  0.0000906,  0.0000913,  0.0000920,  0.0000927,  0.0000934,
     &  0.0000941,  0.0000948,  0.0000955,  0.0000961,  0.0000968,
     &  0.0000974,  0.0000981,  0.0000987,  0.0000994,  0.0001000,
     &  0.0001006,  0.0001012/
      data ((coa(i,j),i=1,62),j= 27, 27)/
     &  0.0000403,  0.0000431,  0.0000456,  0.0000479,  0.0000500,
     &  0.0000520,  0.0000538,  0.0000556,  0.0000573,  0.0000589,
     &  0.0000604,  0.0000619,  0.0000633,  0.0000647,  0.0000661,
     &  0.0000674,  0.0000686,  0.0000699,  0.0000711,  0.0000723,
     &  0.0000734,  0.0000746,  0.0000757,  0.0000768,  0.0000778,
     &  0.0000789,  0.0000799,  0.0000809,  0.0000819,  0.0000829,
     &  0.0000838,  0.0000848,  0.0000857,  0.0000866,  0.0000875,
     &  0.0000884,  0.0000893,  0.0000902,  0.0000910,  0.0000919,
     &  0.0000927,  0.0000935,  0.0000943,  0.0000951,  0.0000959,
     &  0.0000967,  0.0000974,  0.0000982,  0.0000990,  0.0000997,
     &  0.0001004,  0.0001012,  0.0001019,  0.0001026,  0.0001033,
     &  0.0001040,  0.0001047,  0.0001054,  0.0001061,  0.0001067,
     &  0.0001074,  0.0001080/
      data ((coa(i,j),i=1,62),j= 28, 28)/
     &  0.0000426,  0.0000456,  0.0000482,  0.0000507,  0.0000529,
     &  0.0000550,  0.0000570,  0.0000589,  0.0000607,  0.0000624,
     &  0.0000641,  0.0000657,  0.0000672,  0.0000687,  0.0000702,
     &  0.0000716,  0.0000730,  0.0000743,  0.0000756,  0.0000769,
     &  0.0000781,  0.0000794,  0.0000806,  0.0000817,  0.0000829,
     &  0.0000840,  0.0000851,  0.0000862,  0.0000873,  0.0000883,
     &  0.0000893,  0.0000904,  0.0000913,  0.0000923,  0.0000933,
     &  0.0000943,  0.0000952,  0.0000961,  0.0000970,  0.0000979,
     &  0.0000988,  0.0000997,  0.0001006,  0.0001014,  0.0001023,
     &  0.0001031,  0.0001039,  0.0001047,  0.0001055,  0.0001063,
     &  0.0001071,  0.0001079,  0.0001087,  0.0001094,  0.0001102,
     &  0.0001109,  0.0001116,  0.0001124,  0.0001131,  0.0001138,
     &  0.0001145,  0.0001152/
      data ((coa(i,j),i=1,62),j= 29, 29)/
     &  0.0000451,  0.0000482,  0.0000511,  0.0000537,  0.0000561,
     &  0.0000584,  0.0000605,  0.0000626,  0.0000645,  0.0000664,
     &  0.0000682,  0.0000699,  0.0000715,  0.0000732,  0.0000747,
     &  0.0000763,  0.0000777,  0.0000792,  0.0000806,  0.0000820,
     &  0.0000833,  0.0000846,  0.0000859,  0.0000872,  0.0000884,
     &  0.0000896,  0.0000908,  0.0000920,  0.0000931,  0.0000942,
     &  0.0000953,  0.0000964,  0.0000975,  0.0000986,  0.0000996,
     &  0.0001006,  0.0001016,  0.0001026,  0.0001036,  0.0001046,
     &  0.0001055,  0.0001064,  0.0001074,  0.0001083,  0.0001092,
     &  0.0001101,  0.0001110,  0.0001118,  0.0001127,  0.0001135,
     &  0.0001144,  0.0001152,  0.0001160,  0.0001168,  0.0001176,
     &  0.0001184,  0.0001192,  0.0001200,  0.0001207,  0.0001215,
     &  0.0001222,  0.0001230/
      data ((coa(i,j),i=1,62),j= 30, 30)/
     &  0.0000478,  0.0000512,  0.0000543,  0.0000571,  0.0000597,
     &  0.0000621,  0.0000644,  0.0000666,  0.0000687,  0.0000708,
     &  0.0000727,  0.0000746,  0.0000764,  0.0000781,  0.0000798,
     &  0.0000814,  0.0000830,  0.0000846,  0.0000861,  0.0000876,
     &  0.0000891,  0.0000905,  0.0000919,  0.0000932,  0.0000945,
     &  0.0000958,  0.0000971,  0.0000984,  0.0000996,  0.0001008,
     &  0.0001020,  0.0001032,  0.0001043,  0.0001055,  0.0001066,
     &  0.0001077,  0.0001088,  0.0001098,  0.0001109,  0.0001119,
     &  0.0001129,  0.0001139,  0.0001149,  0.0001159,  0.0001168,
     &  0.0001178,  0.0001187,  0.0001197,  0.0001206,  0.0001215,
     &  0.0001224,  0.0001233,  0.0001241,  0.0001250,  0.0001258,
     &  0.0001267,  0.0001275,  0.0001283,  0.0001292,  0.0001300,
     &  0.0001308,  0.0001316/
      data ((coa(i,j),i=1,62),j= 31, 31)/
     &  0.0000508,  0.0000544,  0.0000577,  0.0000607,  0.0000635,
     &  0.0000661,  0.0000686,  0.0000710,  0.0000733,  0.0000754,
     &  0.0000775,  0.0000795,  0.0000815,  0.0000834,  0.0000852,
     &  0.0000870,  0.0000887,  0.0000904,  0.0000920,  0.0000936,
     &  0.0000952,  0.0000967,  0.0000982,  0.0000996,  0.0001011,
     &  0.0001025,  0.0001038,  0.0001052,  0.0001065,  0.0001078,
     &  0.0001091,  0.0001103,  0.0001116,  0.0001128,  0.0001140,
     &  0.0001151,  0.0001163,  0.0001174,  0.0001186,  0.0001197,
     &  0.0001207,  0.0001218,  0.0001229,  0.0001239,  0.0001249,
     &  0.0001260,  0.0001270,  0.0001279,  0.0001289,  0.0001299,
     &  0.0001308,  0.0001318,  0.0001327,  0.0001336,  0.0001317,
     &  0.0001325,  0.0001363,  0.0001372,  0.0001380,  0.0001389,
     &  0.0001397,  0.0001406/
      data ((coa(i,j),i=1,62),j= 32, 32)/
     &  0.0000540,  0.0000579,  0.0000615,  0.0000647,  0.0000677,
     &  0.0000706,  0.0000733,  0.0000758,  0.0000783,  0.0000806,
     &  0.0000829,  0.0000851,  0.0000872,  0.0000892,  0.0000912,
     &  0.0000931,  0.0000950,  0.0000968,  0.0000985,  0.0001003,
     &  0.0001020,  0.0001036,  0.0001052,  0.0001068,  0.0001083,
     &  0.0001098,  0.0001113,  0.0001127,  0.0001142,  0.0001156,
     &  0.0001169,  0.0001183,  0.0001196,  0.0001209,  0.0001222,
     &  0.0001234,  0.0001246,  0.0001259,  0.0001270,  0.0001282,
     &  0.0001294,  0.0001305,  0.0001317,  0.0001328,  0.0001339,
     &  0.0001321,  0.0001360,  0.0001371,  0.0001381,  0.0001391,
     &  0.0001401,  0.0001411,  0.0001421,  0.0001431,  0.0001440,
     &  0.0001450,  0.0001459,  0.0001469,  0.0001478,  0.0001487,
     &  0.0001496,  0.0001505/
      data ((coa(i,j),i=1,62),j= 33, 33)/
     &  0.0000575,  0.0000617,  0.0000655,  0.0000690,  0.0000723,
     &  0.0000754,  0.0000783,  0.0000810,  0.0000837,  0.0000862,
     &  0.0000887,  0.0000910,  0.0000933,  0.0000955,  0.0000976,
     &  0.0000997,  0.0001017,  0.0001036,  0.0001055,  0.0001074,
     &  0.0001092,  0.0001110,  0.0001127,  0.0001144,  0.0001160,
     &  0.0001176,  0.0001192,  0.0001208,  0.0001223,  0.0001238,
     &  0.0001252,  0.0001267,  0.0001281,  0.0001295,  0.0001308,
     &  0.0001322,  0.0001335,  0.0001319,  0.0001360,  0.0001373,
     &  0.0001385,  0.0001397,  0.0001409,  0.0001421,  0.0001433,
     &  0.0001444,  0.0001456,  0.0001467,  0.0001478,  0.0001489,
     &  0.0001499,  0.0001510,  0.0001520,  0.0001531,  0.0001541,
     &  0.0001551,  0.0001561,  0.0001571,  0.0001581,  0.0001590,
     &  0.0001600,  0.0001609/
      data ((coa(i,j),i=1,62),j= 34, 34)/
     &  0.0000613,  0.0000659,  0.0000700,  0.0000738,  0.0000773,
     &  0.0000806,  0.0000838,  0.0000868,  0.0000896,  0.0000924,
     &  0.0000950,  0.0000976,  0.0001000,  0.0001024,  0.0001047,
     &  0.0001069,  0.0001091,  0.0001112,  0.0001132,  0.0001152,
     &  0.0001172,  0.0001191,  0.0001209,  0.0001227,  0.0001245,
     &  0.0001262,  0.0001279,  0.0001296,  0.0001312,  0.0001328,
     &  0.0001344,  0.0001359,  0.0001374,  0.0001389,  0.0001403,
     &  0.0001417,  0.0001432,  0.0001445,  0.0001459,  0.0001472,
     &  0.0001485,  0.0001498,  0.0001511,  0.0001524,  0.0001536,
     &  0.0001548,  0.0001560,  0.0001572,  0.0001584,  0.0001595,
     &  0.0001607,  0.0001618,  0.0001629,  0.0001640,  0.0001651,
     &  0.0001661,  0.0001672,  0.0001682,  0.0001693,  0.0001703,
     &  0.0001713,  0.0001723/
      data ((coa(i,j),i=1,62),j= 35, 35)/
     &  0.0000654,  0.0000703,  0.0000747,  0.0000789,  0.0000827,
     &  0.0000863,  0.0000897,  0.0000929,  0.0000960,  0.0000990,
     &  0.0001018,  0.0001046,  0.0001072,  0.0001098,  0.0001123,
     &  0.0001147,  0.0001170,  0.0001193,  0.0001214,  0.0001236,
     &  0.0001257,  0.0001277,  0.0001297,  0.0001316,  0.0001335,
     &  0.0001325,  0.0001372,  0.0001389,  0.0001407,  0.0001424,
     &  0.0001440,  0.0001457,  0.0001473,  0.0001488,  0.0001504,
     &  0.0001519,  0.0001534,  0.0001548,  0.0001563,  0.0001577,
     &  0.0001591,  0.0001605,  0.0001618,  0.0001631,  0.0001645,
     &  0.0001658,  0.0001670,  0.0001683,  0.0001695,  0.0001707,
     &  0.0001720,  0.0001732,  0.0001743,  0.0001755,  0.0001767,
     &  0.0001778,  0.0001789,  0.0001800,  0.0001811,  0.0001822,
     &  0.0001833,  0.0001844/
      data ((coa(i,j),i=1,62),j= 36, 36)/
     &  0.0000699,  0.0000752,  0.0000800,  0.0000844,  0.0000886,
     &  0.0000925,  0.0000962,  0.0000997,  0.0001030,  0.0001062,
     &  0.0001093,  0.0001123,  0.0001151,  0.0001179,  0.0001205,
     &  0.0001231,  0.0001256,  0.0001280,  0.0001304,  0.0001327,
     &  0.0001321,  0.0001371,  0.0001392,  0.0001413,  0.0001433,
     &  0.0001453,  0.0001472,  0.0001491,  0.0001509,  0.0001527,
     &  0.0001545,  0.0001562,  0.0001579,  0.0001596,  0.0001612,
     &  0.0001629,  0.0001644,  0.0001660,  0.0001675,  0.0001690,
     &  0.0001705,  0.0001720,  0.0001734,  0.0001749,  0.0001762,
     &  0.0001776,  0.0001790,  0.0001803,  0.0001817,  0.0001830,
     &  0.0001842,  0.0001855,  0.0001868,  0.0001880,  0.0001892,
     &  0.0001905,  0.0001917,  0.0001928,  0.0001940,  0.0001952,
     &  0.0001963,  0.0001975/
      data ((coa(i,j),i=1,62),j= 37, 37)/
     &  0.0000748,  0.0000805,  0.0000858,  0.0000906,  0.0000951,
     &  0.0000993,  0.0001033,  0.0001071,  0.0001107,  0.0001142,
     &  0.0001175,  0.0001207,  0.0001238,  0.0001267,  0.0001296,
     &  0.0001323,  0.0001322,  0.0001376,  0.0001401,  0.0001426,
     &  0.0001450,  0.0001473,  0.0001496,  0.0001518,  0.0001539,
     &  0.0001560,  0.0001581,  0.0001601,  0.0001620,  0.0001640,
     &  0.0001659,  0.0001677,  0.0001695,  0.0001713,  0.0001731,
     &  0.0001748,  0.0001765,  0.0001781,  0.0001798,  0.0001814,
     &  0.0001830,  0.0001845,  0.0001861,  0.0001876,  0.0001891,
     &  0.0001905,  0.0001920,  0.0001934,  0.0001948,  0.0001962,
     &  0.0001976,  0.0001990,  0.0002003,  0.0002017,  0.0002030,
     &  0.0002043,  0.0002056,  0.0002068,  0.0002081,  0.0002093,
     &  0.0002106,  0.0002118/
      data ((coa(i,j),i=1,62),j= 38, 38)/
     &  0.0000802,  0.0000863,  0.0000920,  0.0000972,  0.0001021,
     &  0.0001067,  0.0001110,  0.0001151,  0.0001190,  0.0001227,
     &  0.0001263,  0.0001297,  0.0001330,  0.0001362,  0.0001393,
     &  0.0001422,  0.0001451,  0.0001479,  0.0001506,  0.0001532,
     &  0.0001557,  0.0001582,  0.0001606,  0.0001630,  0.0001653,
     &  0.0001675,  0.0001697,  0.0001719,  0.0001740,  0.0001760,
     &  0.0001780,  0.0001800,  0.0001819,  0.0001839,  0.0001857,
     &  0.0001876,  0.0001894,  0.0001911,  0.0001929,  0.0001946,
     &  0.0001963,  0.0001980,  0.0001996,  0.0002012,  0.0002028,
     &  0.0002044,  0.0002060,  0.0002075,  0.0002090,  0.0002105,
     &  0.0002120,  0.0002135,  0.0002149,  0.0002164,  0.0002178,
     &  0.0002192,  0.0002205,  0.0002219,  0.0002233,  0.0002246,
     &  0.0002259,  0.0002273/
      data ((coa(i,j),i=1,62),j= 39, 39)/
     &  0.0000859,  0.0000926,  0.0000987,  0.0001044,  0.0001097,
     &  0.0001146,  0.0001193,  0.0001237,  0.0001279,  0.0001319,
     &  0.0001358,  0.0001395,  0.0001430,  0.0001464,  0.0001497,
     &  0.0001528,  0.0001559,  0.0001589,  0.0001617,  0.0001645,
     &  0.0001673,  0.0001699,  0.0001725,  0.0001750,  0.0001774,
     &  0.0001798,  0.0001822,  0.0001845,  0.0001867,  0.0001889,
     &  0.0001911,  0.0001932,  0.0001953,  0.0001973,  0.0001993,
     &  0.0002013,  0.0002032,  0.0002051,  0.0002070,  0.0002088,
     &  0.0002107,  0.0002124,  0.0002142,  0.0002160,  0.0002177,
     &  0.0002194,  0.0002211,  0.0002227,  0.0002243,  0.0002260,
     &  0.0002276,  0.0002291,  0.0002307,  0.0002322,  0.0002338,
     &  0.0002353,  0.0002368,  0.0002382,  0.0002397,  0.0002412,
     &  0.0002426,  0.0002440/
      data ((coa(i,j),i=1,62),j= 40, 40)/
     &  0.0000922,  0.0000995,  0.0001061,  0.0001122,  0.0001179,
     &  0.0001233,  0.0001283,  0.0001331,  0.0001376,  0.0001419,
     &  0.0001460,  0.0001500,  0.0001538,  0.0001574,  0.0001609,
     &  0.0001643,  0.0001676,  0.0001707,  0.0001738,  0.0001768,
     &  0.0001797,  0.0001825,  0.0001853,  0.0001880,  0.0001906,
     &  0.0001932,  0.0001957,  0.0001981,  0.0002006,  0.0002029,
     &  0.0002052,  0.0002075,  0.0002097,  0.0002119,  0.0002141,
     &  0.0002162,  0.0002183,  0.0002203,  0.0002223,  0.0002243,
     &  0.0002263,  0.0002282,  0.0002301,  0.0002320,  0.0002339,
     &  0.0002357,  0.0002375,  0.0002393,  0.0002411,  0.0002428,
     &  0.0002446,  0.0002463,  0.0002480,  0.0002496,  0.0002513,
     &  0.0002529,  0.0002546,  0.0002562,  0.0002578,  0.0002593,
     &  0.0002609,  0.0002625/
      data ((coa(i,j),i=1,62),j= 41, 41)/
     &  0.0000990,  0.0001069,  0.0001141,  0.0001207,  0.0001268,
     &  0.0001326,  0.0001380,  0.0001431,  0.0001480,  0.0001526,
     &  0.0001570,  0.0001612,  0.0001653,  0.0001692,  0.0001729,
     &  0.0001766,  0.0001801,  0.0001835,  0.0001868,  0.0001900,
     &  0.0001931,  0.0001961,  0.0001991,  0.0002019,  0.0002048,
     &  0.0002075,  0.0002102,  0.0002129,  0.0002154,  0.0002180,
     &  0.0002205,  0.0002229,  0.0002253,  0.0002277,  0.0002300,
     &  0.0002323,  0.0002346,  0.0002368,  0.0002390,  0.0002411,
     &  0.0002432,  0.0002453,  0.0002474,  0.0002494,  0.0002515,
     &  0.0002535,  0.0002554,  0.0002574,  0.0002593,  0.0002612,
     &  0.0002631,  0.0002649,  0.0002668,  0.0002686,  0.0002704,
     &  0.0002722,  0.0002740,  0.0002757,  0.0002775,  0.0002792,
     &  0.0002809,  0.0002826/
      data ((coa(i,j),i=1,62),j= 42, 42)/
     &  0.0001063,  0.0001148,  0.0001226,  0.0001297,  0.0001363,
     &  0.0001425,  0.0001483,  0.0001538,  0.0001590,  0.0001639,
     &  0.0001687,  0.0001732,  0.0001775,  0.0001817,  0.0001857,
     &  0.0001896,  0.0001933,  0.0001970,  0.0002005,  0.0002039,
     &  0.0002073,  0.0002105,  0.0002137,  0.0002168,  0.0002198,
     &  0.0002228,  0.0002257,  0.0002286,  0.0002314,  0.0002341,
     &  0.0002368,  0.0002394,  0.0002420,  0.0002446,  0.0002471,
     &  0.0002496,  0.0002520,  0.0002544,  0.0002568,  0.0002591,
     &  0.0002615,  0.0002637,  0.0002660,  0.0002682,  0.0002704,
     &  0.0002726,  0.0002747,  0.0002768,  0.0002789,  0.0002810,
     &  0.0002831,  0.0002851,  0.0002871,  0.0002891,  0.0002911,
     &  0.0002930,  0.0002950,  0.0002969,  0.0002988,  0.0003007,
     &  0.0003025,  0.0003044/
      data ((coa(i,j),i=1,62),j= 43, 43)/
     &  0.0001141,  0.0001233,  0.0001316,  0.0001393,  0.0001464,
     &  0.0001531,  0.0001593,  0.0001652,  0.0001707,  0.0001760,
     &  0.0001811,  0.0001859,  0.0001905,  0.0001950,  0.0001993,
     &  0.0002035,  0.0002075,  0.0002114,  0.0002152,  0.0002189,
     &  0.0002225,  0.0002260,  0.0002294,  0.0002328,  0.0002360,
     &  0.0002393,  0.0002424,  0.0002455,  0.0002485,  0.0002515,
     &  0.0002544,  0.0002573,  0.0002601,  0.0002629,  0.0002656,
     &  0.0002683,  0.0002709,  0.0002736,  0.0002762,  0.0002787,
     &  0.0002812,  0.0002837,  0.0002862,  0.0002886,  0.0002910,
     &  0.0002934,  0.0002957,  0.0002980,  0.0003003,  0.0003026,
     &  0.0003048,  0.0003071,  0.0003093,  0.0003114,  0.0003136,
     &  0.0003157,  0.0003179,  0.0003200,  0.0003221,  0.0003241,
     &  0.0003262,  0.0003282/
      data ((coa(i,j),i=1,62),j= 44, 44)/
     &  0.0001224,  0.0001323,  0.0001413,  0.0001496,  0.0001572,
     &  0.0001643,  0.0001709,  0.0001772,  0.0001832,  0.0001888,
     &  0.0001943,  0.0001994,  0.0002044,  0.0002092,  0.0002138,
     &  0.0002183,  0.0002226,  0.0002269,  0.0002309,  0.0002349,
     &  0.0002388,  0.0002426,  0.0002463,  0.0002499,  0.0002535,
     &  0.0002570,  0.0002604,  0.0002637,  0.0002670,  0.0002702,
     &  0.0002734,  0.0002765,  0.0002796,  0.0002826,  0.0002856,
     &  0.0002886,  0.0002915,  0.0002943,  0.0002972,  0.0002999,
     &  0.0003027,  0.0003054,  0.0003081,  0.0003108,  0.0003134,
     &  0.0003160,  0.0003185,  0.0003211,  0.0003236,  0.0003261,
     &  0.0003286,  0.0003310,  0.0003334,  0.0003358,  0.0003382,
     &  0.0003405,  0.0003428,  0.0003451,  0.0003474,  0.0003497,
     &  0.0003519,  0.0003542/
      data ((coa(i,j),i=1,62),j= 45, 45)/
     &  0.0001312,  0.0001419,  0.0001515,  0.0001603,  0.0001685,
     &  0.0001761,  0.0001832,  0.0001899,  0.0001963,  0.0002024,
     &  0.0002082,  0.0002138,  0.0002191,  0.0002243,  0.0002292,
     &  0.0002341,  0.0002387,  0.0002433,  0.0002477,  0.0002520,
     &  0.0002562,  0.0002603,  0.0002644,  0.0002683,  0.0002722,
     &  0.0002759,  0.0002796,  0.0002833,  0.0002869,  0.0002904,
     &  0.0002939,  0.0002973,  0.0003006,  0.0003039,  0.0003072,
     &  0.0003104,  0.0003136,  0.0003167,  0.0003198,  0.0003228,
     &  0.0003259,  0.0003288,  0.0003318,  0.0003347,  0.0003376,
     &  0.0003404,  0.0003432,  0.0003460,  0.0003487,  0.0003515,
     &  0.0003542,  0.0003568,  0.0003595,  0.0003621,  0.0003647,
     &  0.0003673,  0.0003698,  0.0003724,  0.0003749,  0.0003773,
     &  0.0003798,  0.0003822/
      data ((coa(i,j),i=1,62),j= 46, 46)/
     &  0.0001406,  0.0001520,  0.0001623,  0.0001718,  0.0001805,
     &  0.0001886,  0.0001963,  0.0002035,  0.0002103,  0.0002168,
     &  0.0002231,  0.0002291,  0.0002348,  0.0002404,  0.0002458,
     &  0.0002510,  0.0002561,  0.0002610,  0.0002658,  0.0002705,
     &  0.0002750,  0.0002795,  0.0002839,  0.0002882,  0.0002924,
     &  0.0002965,  0.0003005,  0.0003045,  0.0003084,  0.0003123,
     &  0.0003161,  0.0003198,  0.0003235,  0.0003271,  0.0003307,
     &  0.0003342,  0.0003376,  0.0003411,  0.0003445,  0.0003478,
     &  0.0003511,  0.0003544,  0.0003576,  0.0003608,  0.0003639,
     &  0.0003670,  0.0003701,  0.0003731,  0.0003762,  0.0003791,
     &  0.0003821,  0.0003850,  0.0003879,  0.0003908,  0.0003936,
     &  0.0003965,  0.0003992,  0.0004020,  0.0004047,  0.0004075,
     &  0.0004102,  0.0004128/
      data ((coa(i,j),i=1,62),j= 47, 47)/
     &  0.0001506,  0.0001628,  0.0001739,  0.0001840,  0.0001934,
     &  0.0002021,  0.0002103,  0.0002180,  0.0002254,  0.0002324,
     &  0.0002391,  0.0002456,  0.0002518,  0.0002579,  0.0002637,
     &  0.0002694,  0.0002749,  0.0002802,  0.0002854,  0.0002905,
     &  0.0002955,  0.0003004,  0.0003052,  0.0003099,  0.0003145,
     &  0.0003190,  0.0003234,  0.0003278,  0.0003320,  0.0003362,
     &  0.0003404,  0.0003445,  0.0003485,  0.0003524,  0.0003564,
     &  0.0003602,  0.0003640,  0.0003678,  0.0003715,  0.0003751,
     &  0.0003787,  0.0003823,  0.0003858,  0.0003893,  0.0003928,
     &  0.0003962,  0.0003995,  0.0004029,  0.0004062,  0.0004094,
     &  0.0004127,  0.0004159,  0.0004190,  0.0004222,  0.0004253,
     &  0.0004283,  0.0004314,  0.0004344,  0.0004374,  0.0004404,
     &  0.0004433,  0.0004462/
      data ((coa(i,j),i=1,62),j= 48, 48)/
     &  0.0001613,  0.0001744,  0.0001863,  0.0001971,  0.0002072,
     &  0.0002165,  0.0002254,  0.0002337,  0.0002417,  0.0002493,
     &  0.0002565,  0.0002636,  0.0002703,  0.0002769,  0.0002832,
     &  0.0002894,  0.0002954,  0.0003013,  0.0003070,  0.0003125,
     &  0.0003180,  0.0003233,  0.0003286,  0.0003337,  0.0003387,
     &  0.0003436,  0.0003485,  0.0003533,  0.0003579,  0.0003625,
     &  0.0003671,  0.0003716,  0.0003760,  0.0003803,  0.0003846,
     &  0.0003888,  0.0003929,  0.0003971,  0.0004011,  0.0004051,
     &  0.0004091,  0.0004130,  0.0004168,  0.0004206,  0.0004244,
     &  0.0004281,  0.0004318,  0.0004354,  0.0004390,  0.0004426,
     &  0.0004461,  0.0004496,  0.0004530,  0.0004565,  0.0004598,
     &  0.0004632,  0.0004665,  0.0004698,  0.0004730,  0.0004763,
     &  0.0004795,  0.0004826/
      data ((coa(i,j),i=1,62),j= 49, 49)/
     &  0.0001728,  0.0001868,  0.0001996,  0.0002112,  0.0002220,
     &  0.0002321,  0.0002417,  0.0002507,  0.0002593,  0.0002676,
     &  0.0002755,  0.0002831,  0.0002905,  0.0002977,  0.0003046,
     &  0.0003113,  0.0003179,  0.0003243,  0.0003305,  0.0003366,
     &  0.0003426,  0.0003484,  0.0003542,  0.0003598,  0.0003653,
     &  0.0003707,  0.0003760,  0.0003812,  0.0003863,  0.0003914,
     &  0.0003963,  0.0004012,  0.0004060,  0.0004108,  0.0004154,
     &  0.0004201,  0.0004246,  0.0004291,  0.0004335,  0.0004379,
     &  0.0004422,  0.0004464,  0.0004506,  0.0004548,  0.0004589,
     &  0.0004629,  0.0004669,  0.0004709,  0.0004748,  0.0004787,
     &  0.0004825,  0.0004863,  0.0004900,  0.0004937,  0.0004974,
     &  0.0005010,  0.0005046,  0.0005082,  0.0005117,  0.0005152,
     &  0.0005187,  0.0005221/
      data ((coa(i,j),i=1,62),j= 50, 50)/
     &  0.0001851,  0.0002003,  0.0002139,  0.0002265,  0.0002382,
     &  0.0002491,  0.0002595,  0.0002693,  0.0002787,  0.0002877,
     &  0.0002963,  0.0003047,  0.0003127,  0.0003205,  0.0003281,
     &  0.0003355,  0.0003427,  0.0003497,  0.0003565,  0.0003632,
     &  0.0003697,  0.0003761,  0.0003824,  0.0003885,  0.0003945,
     &  0.0004004,  0.0004062,  0.0004119,  0.0004175,  0.0004230,
     &  0.0004285,  0.0004338,  0.0004390,  0.0004442,  0.0004493,
     &  0.0004543,  0.0004593,  0.0004641,  0.0004689,  0.0004737,
     &  0.0004784,  0.0004830,  0.0004875,  0.0004920,  0.0004965,
     &  0.0005009,  0.0005052,  0.0005095,  0.0005138,  0.0005179,
     &  0.0005221,  0.0005262,  0.0005302,  0.0005342,  0.0005268,
     &  0.0005421,  0.0005460,  0.0005499,  0.0005537,  0.0005574,
     &  0.0005612,  0.0005649/
      data ((coa(i,j),i=1,62),j= 51, 51)/
     &  0.0001985,  0.0002149,  0.0002297,  0.0002433,  0.0002559,
     &  0.0002679,  0.0002791,  0.0002898,  0.0003001,  0.0003099,
     &  0.0003193,  0.0003285,  0.0003373,  0.0003459,  0.0003542,
     &  0.0003622,  0.0003701,  0.0003778,  0.0003853,  0.0003926,
     &  0.0003997,  0.0004067,  0.0004135,  0.0004202,  0.0004268,
     &  0.0004333,  0.0004396,  0.0004458,  0.0004519,  0.0004579,
     &  0.0004638,  0.0004696,  0.0004753,  0.0004809,  0.0004864,
     &  0.0004919,  0.0004972,  0.0005025,  0.0005077,  0.0005129,
     &  0.0005179,  0.0005229,  0.0005279,  0.0005327,  0.0005375,
     &  0.0005423,  0.0005470,  0.0005516,  0.0005562,  0.0005607,
     &  0.0005652,  0.0005696,  0.0005739,  0.0005782,  0.0005825,
     &  0.0005867,  0.0005909,  0.0005951,  0.0005991,  0.0006032,
     &  0.0006072,  0.0006112/
      data ((coa(i,j),i=1,62),j= 52, 52)/
     &  0.0002132,  0.0002309,  0.0002469,  0.0002617,  0.0002755,
     &  0.0002885,  0.0003008,  0.0003125,  0.0003237,  0.0003345,
     &  0.0003449,  0.0003549,  0.0003645,  0.0003739,  0.0003830,
     &  0.0003918,  0.0004004,  0.0004088,  0.0004170,  0.0004250,
     &  0.0004328,  0.0004404,  0.0004478,  0.0004551,  0.0004623,
     &  0.0004693,  0.0004762,  0.0004829,  0.0004895,  0.0004960,
     &  0.0005024,  0.0005087,  0.0005149,  0.0005210,  0.0005269,
     &  0.0005328,  0.0005272,  0.0005443,  0.0005500,  0.0005555,
     &  0.0005609,  0.0005663,  0.0005717,  0.0005769,  0.0005821,
     &  0.0005872,  0.0005922,  0.0005972,  0.0006021,  0.0006070,
     &  0.0006118,  0.0006165,  0.0006212,  0.0006258,  0.0006304,
     &  0.0006349,  0.0006394,  0.0006438,  0.0006482,  0.0006525,
     &  0.0006568,  0.0006611/
      data ((coa(i,j),i=1,62),j= 53, 53)/
     &  0.0002293,  0.0002485,  0.0002660,  0.0002821,  0.0002972,
     &  0.0003114,  0.0003249,  0.0003377,  0.0003500,  0.0003618,
     &  0.0003732,  0.0003841,  0.0003947,  0.0004049,  0.0004149,
     &  0.0004245,  0.0004339,  0.0004430,  0.0004520,  0.0004606,
     &  0.0004691,  0.0004774,  0.0004855,  0.0004934,  0.0005012,
     &  0.0005087,  0.0005162,  0.0005235,  0.0005306,  0.0005377,
     &  0.0005446,  0.0005513,  0.0005580,  0.0005646,  0.0005710,
     &  0.0005773,  0.0005836,  0.0005897,  0.0005957,  0.0006017,
     &  0.0006076,  0.0006134,  0.0006191,  0.0006247,  0.0006302,
     &  0.0006357,  0.0006411,  0.0006464,  0.0006517,  0.0006569,
     &  0.0006620,  0.0006671,  0.0006721,  0.0006771,  0.0006820,
     &  0.0006868,  0.0006916,  0.0006963,  0.0007010,  0.0007056,
     &  0.0007102,  0.0007147/
      data ((coa(i,j),i=1,62),j= 54, 54)/
     &  0.0002471,  0.0002680,  0.0002871,  0.0003048,  0.0003214,
     &  0.0003369,  0.0003517,  0.0003658,  0.0003792,  0.0003921,
     &  0.0004045,  0.0004165,  0.0004281,  0.0004392,  0.0004501,
     &  0.0004606,  0.0004708,  0.0004807,  0.0004903,  0.0004998,
     &  0.0005089,  0.0005179,  0.0005267,  0.0005352,  0.0005436,
     &  0.0005518,  0.0005598,  0.0005677,  0.0005754,  0.0005829,
     &  0.0005903,  0.0005976,  0.0006048,  0.0006118,  0.0006187,
     &  0.0006255,  0.0006322,  0.0006388,  0.0006453,  0.0006516,
     &  0.0006579,  0.0006641,  0.0006702,  0.0006762,  0.0006822,
     &  0.0006880,  0.0006938,  0.0006995,  0.0007051,  0.0007106,
     &  0.0007161,  0.0007215,  0.0007269,  0.0007321,  0.0007374,
     &  0.0007425,  0.0007476,  0.0007527,  0.0007576,  0.0007626,
     &  0.0007674,  0.0007723/
      data ((coa(i,j),i=1,62),j= 55, 55)/
     &  0.0002669,  0.0002898,  0.0003107,  0.0003300,  0.0003482,
     &  0.0003653,  0.0003815,  0.0003969,  0.0004116,  0.0004257,
     &  0.0004392,  0.0004522,  0.0004648,  0.0004769,  0.0004887,
     &  0.0005001,  0.0005111,  0.0005218,  0.0005323,  0.0005425,
     &  0.0005524,  0.0005620,  0.0005714,  0.0005807,  0.0005897,
     &  0.0005985,  0.0006071,  0.0006155,  0.0006238,  0.0006319,
     &  0.0006398,  0.0006476,  0.0006553,  0.0006628,  0.0006702,
     &  0.0006775,  0.0006846,  0.0006917,  0.0006986,  0.0007054,
     &  0.0007121,  0.0007187,  0.0007252,  0.0007316,  0.0007379,
     &  0.0007442,  0.0007503,  0.0007564,  0.0007624,  0.0007683,
     &  0.0007741,  0.0007798,  0.0007855,  0.0007911,  0.0007967,
     &  0.0008022,  0.0008076,  0.0008129,  0.0008182,  0.0008235,
     &  0.0008286,  0.0008337/
      data ((coa(i,j),i=1,62),j= 56, 56)/
     &  0.0002889,  0.0003140,  0.0003369,  0.0003582,  0.0003780,
     &  0.0003967,  0.0004144,  0.0004312,  0.0004473,  0.0004626,
     &  0.0004773,  0.0004914,  0.0005050,  0.0005182,  0.0005309,
     &  0.0005432,  0.0005551,  0.0005666,  0.0005779,  0.0005888,
     &  0.0005995,  0.0006098,  0.0006200,  0.0006298,  0.0006395,
     &  0.0006489,  0.0006581,  0.0006672,  0.0006760,  0.0006847,
     &  0.0006932,  0.0007015,  0.0007097,  0.0007177,  0.0007256,
     &  0.0007333,  0.0007409,  0.0007484,  0.0007558,  0.0007630,
     &  0.0007702,  0.0007772,  0.0007841,  0.0007909,  0.0007976,
     &  0.0008043,  0.0008108,  0.0008172,  0.0008236,  0.0008298,
     &  0.0008360,  0.0008421,  0.0008481,  0.0008541,  0.0008600,
     &  0.0008658,  0.0008715,  0.0008772,  0.0008828,  0.0008883,
     &  0.0008938,  0.0008992/
      data ((coa(i,j),i=1,62),j= 57, 57)/
     &  0.0003135,  0.0003410,  0.0003662,  0.0003895,  0.0004112,
     &  0.0004316,  0.0004509,  0.0004692,  0.0004866,  0.0005032,
     &  0.0005191,  0.0005344,  0.0005491,  0.0005632,  0.0005769,
     &  0.0005901,  0.0006029,  0.0006153,  0.0006274,  0.0006391,
     &  0.0006505,  0.0006616,  0.0006725,  0.0006830,  0.0006933,
     &  0.0007034,  0.0007132,  0.0007229,  0.0007323,  0.0007415,
     &  0.0007506,  0.0007595,  0.0007682,  0.0007767,  0.0007851,
     &  0.0007933,  0.0008014,  0.0008093,  0.0008172,  0.0008249,
     &  0.0008324,  0.0008399,  0.0008472,  0.0008544,  0.0008615,
     &  0.0008685,  0.0008755,  0.0008823,  0.0008890,  0.0008956,
     &  0.0009021,  0.0009086,  0.0009149,  0.0009212,  0.0009274,
     &  0.0009335,  0.0009396,  0.0009455,  0.0009514,  0.0009573,
     &  0.0009630,  0.0009687/
      data ((coa(i,j),i=1,62),j= 58, 58)/
     &  0.0003409,  0.0003711,  0.0003987,  0.0004241,  0.0004478,
     &  0.0004700,  0.0004909,  0.0005107,  0.0005295,  0.0005474,
     &  0.0005645,  0.0005810,  0.0005968,  0.0006120,  0.0006267,
     &  0.0006408,  0.0006545,  0.0006678,  0.0006807,  0.0006932,
     &  0.0007054,  0.0007173,  0.0007288,  0.0007401,  0.0007510,
     &  0.0007618,  0.0007722,  0.0007825,  0.0007925,  0.0008023,
     &  0.0008119,  0.0008213,  0.0008306,  0.0008396,  0.0008485,
     &  0.0008572,  0.0008658,  0.0008742,  0.0008824,  0.0008906,
     &  0.0008986,  0.0009064,  0.0009142,  0.0009218,  0.0009293,
     &  0.0009367,  0.0009439,  0.0009511,  0.0009582,  0.0009652,
     &  0.0009720,  0.0009788,  0.0009855,  0.0009921,  0.0009986,
     &  0.0010050,  0.0010113,  0.0010176,  0.0010238,  0.0010299,
     &  0.0010359,  0.0010419/
      data ((coa(i,j),i=1,62),j= 59, 59)/
     &  0.0003715,  0.0004046,  0.0004346,  0.0004623,  0.0004880,
     &  0.0005120,  0.0005346,  0.0005560,  0.0005762,  0.0005955,
     &  0.0006139,  0.0006315,  0.0006485,  0.0006648,  0.0006804,
     &  0.0006956,  0.0007102,  0.0007244,  0.0007382,  0.0007515,
     &  0.0007645,  0.0007771,  0.0007893,  0.0008012,  0.0008129,
     &  0.0008243,  0.0008354,  0.0008463,  0.0008569,  0.0008673,
     &  0.0008774,  0.0008874,  0.0008971,  0.0009067,  0.0009160,
     &  0.0009252,  0.0009342,  0.0009431,  0.0009518,  0.0009604,
     &  0.0009688,  0.0009770,  0.0009851,  0.0009931,  0.0010010,
     &  0.0010088,  0.0010164,  0.0010239,  0.0010313,  0.0010386,
     &  0.0010458,  0.0010529,  0.0010598,  0.0010667,  0.0010735,
     &  0.0010802,  0.0010869,  0.0010934,  0.0010998,  0.0011062,
     &  0.0011125,  0.0011187/
      data ((coa(i,j),i=1,62),j= 60, 60)/
     &  0.0004055,  0.0004415,  0.0004742,  0.0005042,  0.0005320,
     &  0.0005579,  0.0005822,  0.0006052,  0.0006269,  0.0006476,
     &  0.0006673,  0.0006862,  0.0007042,  0.0007216,  0.0007383,
     &  0.0007545,  0.0007701,  0.0007851,  0.0007997,  0.0008139,
     &  0.0008276,  0.0008410,  0.0008540,  0.0008666,  0.0008789,
     &  0.0008910,  0.0009027,  0.0009141,  0.0009253,  0.0009362,
     &  0.0009469,  0.0009574,  0.0009676,  0.0009777,  0.0009875,
     &  0.0009972,  0.0010066,  0.0010159,  0.0010250,  0.0010339,
     &  0.0010427,  0.0010514,  0.0010599,  0.0010682,  0.0010764,
     &  0.0010845,  0.0010925,  0.0011003,  0.0011080,  0.0011156,
     &  0.0011231,  0.0011304,  0.0011377,  0.0011449,  0.0011519,
     &  0.0011589,  0.0011658,  0.0011726,  0.0011793,  0.0011859,
     &  0.0011924,  0.0011989/
      data ((coa(i,j),i=1,62),j= 61, 61)/
     &  0.0004429,  0.0004821,  0.0005175,  0.0005499,  0.0005798,
     &  0.0006076,  0.0006337,  0.0006583,  0.0006816,  0.0007037,
     &  0.0007247,  0.0007448,  0.0007640,  0.0007825,  0.0008003,
     &  0.0008174,  0.0008339,  0.0008499,  0.0008653,  0.0008803,
     &  0.0008948,  0.0009089,  0.0009226,  0.0009359,  0.0009488,
     &  0.0009615,  0.0009738,  0.0009858,  0.0009975,  0.0010089,
     &  0.0010201,  0.0010311,  0.0010418,  0.0010523,  0.0010625,
     &  0.0010726,  0.0010825,  0.0010921,  0.0011016,  0.0011109,
     &  0.0011201,  0.0011291,  0.0011379,  0.0011466,  0.0011551,
     &  0.0011635,  0.0011718,  0.0011799,  0.0011879,  0.0011958,
     &  0.0012035,  0.0012112,  0.0012187,  0.0012261,  0.0012335,
     &  0.0012407,  0.0012478,  0.0012548,  0.0012618,  0.0012686,
     &  0.0012754,  0.0012821/
      data ((coa(i,j),i=1,62),j= 62, 62)/
     &  0.0004840,  0.0005264,  0.0005646,  0.0005994,  0.0006316,
     &  0.0006614,  0.0006893,  0.0007155,  0.0007403,  0.0007638,
     &  0.0007862,  0.0008075,  0.0008279,  0.0008475,  0.0008663,
     &  0.0008844,  0.0009018,  0.0009186,  0.0009349,  0.0009506,
     &  0.0009658,  0.0009806,  0.0009950,  0.0010089,  0.0010225,
     &  0.0010356,  0.0010485,  0.0010610,  0.0010733,  0.0010852,
     &  0.0010968,  0.0011082,  0.0011194,  0.0011303,  0.0011409,
     &  0.0011514,  0.0011616,  0.0011717,  0.0011815,  0.0011912,
     &  0.0012007,  0.0012100,  0.0012191,  0.0012281,  0.0012370,
     &  0.0012457,  0.0012542,  0.0012627,  0.0012709,  0.0012791,
     &  0.0012871,  0.0012951,  0.0013029,  0.0013106,  0.0013181,
     &  0.0013256,  0.0013330,  0.0013403,  0.0013475,  0.0013546,
     &  0.0013616,  0.0013685/
      data ((coa(i,j),i=1,62),j= 63, 63)/
     &  0.0005290,  0.0005747,  0.0006157,  0.0006530,  0.0006874,
     &  0.0007192,  0.0007490,  0.0007769,  0.0008032,  0.0008281,
     &  0.0008518,  0.0008743,  0.0008959,  0.0009165,  0.0009362,
     &  0.0009552,  0.0009735,  0.0009911,  0.0010082,  0.0010246,
     &  0.0010405,  0.0010559,  0.0010709,  0.0010854,  0.0010995,
     &  0.0011132,  0.0011266,  0.0011396,  0.0011523,  0.0011647,
     &  0.0011768,  0.0011886,  0.0012002,  0.0012115,  0.0012225,
     &  0.0012334,  0.0012440,  0.0012544,  0.0012646,  0.0012746,
     &  0.0012844,  0.0012941,  0.0013036,  0.0013129,  0.0013220,
     &  0.0013310,  0.0013399,  0.0013486,  0.0013572,  0.0013657,
     &  0.0013740,  0.0013823,  0.0013904,  0.0013983,  0.0014062,
     &  0.0014140,  0.0014217,  0.0014292,  0.0014367,  0.0014441,
     &  0.0014514,  0.0014586/
      data ((coa(i,j),i=1,62),j= 64, 64)/
     &  0.0005778,  0.0006269,  0.0006708,  0.0007107,  0.0007473,
     &  0.0007812,  0.0008127,  0.0008423,  0.0008701,  0.0008964,
     &  0.0009213,  0.0009450,  0.0009676,  0.0009892,  0.0010099,
     &  0.0010297,  0.0010488,  0.0010671,  0.0010848,  0.0011020,
     &  0.0011185,  0.0011345,  0.0011500,  0.0011651,  0.0011798,
     &  0.0011940,  0.0012078,  0.0012213,  0.0012345,  0.0012473,
     &  0.0012599,  0.0012721,  0.0012841,  0.0012958,  0.0013073,
     &  0.0013185,  0.0013295,  0.0013403,  0.0013509,  0.0013612,
     &  0.0013714,  0.0013815,  0.0013913,  0.0014010,  0.0014105,
     &  0.0014199,  0.0014291,  0.0014382,  0.0014471,  0.0014559,
     &  0.0014646,  0.0014732,  0.0014816,  0.0014900,  0.0014982,
     &  0.0015063,  0.0015143,  0.0015222,  0.0015300,  0.0015377,
     &  0.0015453,  0.0015528/
      data ((coa(i,j),i=1,62),j= 65, 65)/
     &  0.0006307,  0.0006832,  0.0007301,  0.0007725,  0.0008114,
     &  0.0008472,  0.0008805,  0.0009116,  0.0009409,  0.0009684,
     &  0.0009945,  0.0010193,  0.0010428,  0.0010653,  0.0010868,
     &  0.0011075,  0.0011273,  0.0011464,  0.0011647,  0.0011825,
     &  0.0011996,  0.0012162,  0.0012323,  0.0012480,  0.0012631,
     &  0.0012779,  0.0012922,  0.0013062,  0.0013199,  0.0013332,
     &  0.0013462,  0.0013589,  0.0013713,  0.0013835,  0.0013954,
     &  0.0014071,  0.0014186,  0.0014298,  0.0014408,  0.0014516,
     &  0.0014623,  0.0014727,  0.0014830,  0.0014931,  0.0015030,
     &  0.0015128,  0.0015225,  0.0015319,  0.0015413,  0.0015505,
     &  0.0015596,  0.0015686,  0.0015774,  0.0015862,  0.0015948,
     &  0.0016033,  0.0016117,  0.0016200,  0.0016282,  0.0016363,
     &  0.0016443,  0.0016522/
      data ((coa(i,j),i=1,62),j= 66, 66)/
     &  0.0006876,  0.0007436,  0.0007934,  0.0008383,  0.0008793,
     &  0.0009170,  0.0009520,  0.0009846,  0.0010150,  0.0010439,
     &  0.0010710,  0.0010968,  0.0011213,  0.0011446,  0.0011669,
     &  0.0011883,  0.0012089,  0.0012287,  0.0012477,  0.0012661,
     &  0.0012839,  0.0013011,  0.0013178,  0.0013340,  0.0013498,
     &  0.0013651,  0.0013800,  0.0013946,  0.0014088,  0.0014227,
     &  0.0014362,  0.0014495,  0.0014624,  0.0014751,  0.0014876,
     &  0.0014998,  0.0015117,  0.0015235,  0.0015350,  0.0015464,
     &  0.0015575,  0.0015685,  0.0015792,  0.0015899,  0.0016003,
     &  0.0016106,  0.0016207,  0.0016307,  0.0016405,  0.0016502,
     &  0.0016598,  0.0016693,  0.0016786,  0.0016878,  0.0016969,
     &  0.0017059,  0.0017147,  0.0017235,  0.0017322,  0.0017407,
     &  0.0017492,  0.0017575/
      data ((coa(i,j),i=1,62),j= 67, 67)/
     &  0.0007485,  0.0008080,  0.0008606,  0.0009079,  0.0009509,
     &  0.0009904,  0.0010269,  0.0010608,  0.0010926,  0.0011225,
     &  0.0011507,  0.0011774,  0.0012028,  0.0012270,  0.0012501,
     &  0.0012723,  0.0012936,  0.0013142,  0.0013339,  0.0013531,
     &  0.0013716,  0.0013895,  0.0014069,  0.0014238,  0.0014402,
     &  0.0014562,  0.0014718,  0.0014870,  0.0015019,  0.0015164,
     &  0.0015306,  0.0015445,  0.0015581,  0.0015714,  0.0015845,
     &  0.0015973,  0.0016099,  0.0016223,  0.0016344,  0.0016464,
     &  0.0016581,  0.0016697,  0.0016811,  0.0016922,  0.0017033,
     &  0.0017141,  0.0017249,  0.0017354,  0.0017458,  0.0017561,
     &  0.0017662,  0.0017762,  0.0017861,  0.0017959,  0.0018055,
     &  0.0018150,  0.0018244,  0.0018337,  0.0018429,  0.0018520,
     &  0.0018610,  0.0018698/
      data ((coa(i,j),i=1,62),j= 68, 68)/
     &  0.0008135,  0.0008762,  0.0009315,  0.0009811,  0.0010259,
     &  0.0010670,  0.0011050,  0.0011402,  0.0011732,  0.0012042,
     &  0.0012334,  0.0012611,  0.0012874,  0.0013126,  0.0013366,
     &  0.0013597,  0.0013819,  0.0014033,  0.0014239,  0.0014439,
     &  0.0014632,  0.0014820,  0.0015002,  0.0015179,  0.0015351,
     &  0.0015519,  0.0015683,  0.0015843,  0.0016000,  0.0016153,
     &  0.0016302,  0.0016449,  0.0016592,  0.0016733,  0.0016871,
     &  0.0017007,  0.0017140,  0.0017271,  0.0017400,  0.0017526,
     &  0.0017651,  0.0017773,  0.0017894,  0.0018012,  0.0018129,
     &  0.0018245,  0.0018358,  0.0018471,  0.0018581,  0.0018690,
     &  0.0018798,  0.0018904,  0.0019009,  0.0019113,  0.0019215,
     &  0.0019316,  0.0019416,  0.0019515,  0.0019613,  0.0019709,
     &  0.0019805,  0.0019899/
      data ((coa(i,j),i=1,62),j= 69, 69)/
     &  0.0008823,  0.0009481,  0.0010059,  0.0010575,  0.0011042,
     &  0.0011468,  0.0011862,  0.0012227,  0.0012569,  0.0012891,
     &  0.0013195,  0.0013483,  0.0013757,  0.0014020,  0.0014271,
     &  0.0014512,  0.0014744,  0.0014968,  0.0015185,  0.0015395,
     &  0.0015598,  0.0015795,  0.0015987,  0.0016174,  0.0016356,
     &  0.0016533,  0.0016707,  0.0016876,  0.0017041,  0.0017203,
     &  0.0017362,  0.0017517,  0.0017669,  0.0017819,  0.0017965,
     &  0.0018109,  0.0018251,  0.0018390,  0.0018527,  0.0018661,
     &  0.0018793,  0.0018924,  0.0019052,  0.0019178,  0.0019303,
     &  0.0019425,  0.0019546,  0.0019665,  0.0019783,  0.0019899,
     &  0.0020014,  0.0020127,  0.0020238,  0.0020348,  0.0020457,
     &  0.0020565,  0.0020671,  0.0020776,  0.0020880,  0.0020983,
     &  0.0021084,  0.0021185/
      data ((coa(i,j),i=1,62),j= 70, 70)/
     &  0.0009546,  0.0010233,  0.0010834,  0.0011370,  0.0011854,
     &  0.0012297,  0.0012705,  0.0013085,  0.0013441,  0.0013776,
     &  0.0014093,  0.0014395,  0.0014682,  0.0014957,  0.0015221,
     &  0.0015475,  0.0015720,  0.0015956,  0.0016185,  0.0016406,
     &  0.0016621,  0.0016830,  0.0017033,  0.0017231,  0.0017424,
     &  0.0017613,  0.0017797,  0.0017976,  0.0018152,  0.0018324,
     &  0.0018493,  0.0018658,  0.0018820,  0.0018979,  0.0019135,
     &  0.0019288,  0.0019439,  0.0019587,  0.0019732,  0.0019875,
     &  0.0020016,  0.0020155,  0.0020291,  0.0020425,  0.0020558,
     &  0.0020688,  0.0020817,  0.0020944,  0.0021069,  0.0021192,
     &  0.0021314,  0.0021434,  0.0021095,  0.0021670,  0.0021786,
     &  0.0021900,  0.0022013,  0.0022125,  0.0022235,  0.0022344,
     &  0.0022452,  0.0022558/
      data ((coa(i,j),i=1,62),j= 71, 71)/
     &  0.0010302,  0.0011016,  0.0011640,  0.0012195,  0.0012698,
     &  0.0013159,  0.0013584,  0.0013981,  0.0014354,  0.0014705,
     &  0.0015038,  0.0015355,  0.0015658,  0.0015948,  0.0016227,
     &  0.0016496,  0.0016755,  0.0017005,  0.0017248,  0.0017483,
     &  0.0017712,  0.0017934,  0.0018150,  0.0018360,  0.0018566,
     &  0.0018766,  0.0018962,  0.0019153,  0.0019340,  0.0019524,
     &  0.0019703,  0.0019879,  0.0020052,  0.0020221,  0.0020387,
     &  0.0020550,  0.0020710,  0.0020867,  0.0021022,  0.0021174,
     &  0.0021324,  0.0021471,  0.0021159,  0.0021759,  0.0021900,
     &  0.0022039,  0.0022175,  0.0022310,  0.0022443,  0.0022574,
     &  0.0022703,  0.0022830,  0.0022956,  0.0023080,  0.0023203,
     &  0.0023324,  0.0023443,  0.0023562,  0.0023678,  0.0023794,
     &  0.0023908,  0.0024020/
      data ((coa(i,j),i=1,62),j= 72, 72)/
     &  0.0011087,  0.0011828,  0.0012476,  0.0013054,  0.0013579,
     &  0.0014060,  0.0014506,  0.0014923,  0.0015315,  0.0015685,
     &  0.0016038,  0.0016373,  0.0016694,  0.0017002,  0.0017298,
     &  0.0017583,  0.0017859,  0.0018125,  0.0018384,  0.0018634,
     &  0.0018877,  0.0019114,  0.0019344,  0.0019568,  0.0019787,
     &  0.0020000,  0.0020209,  0.0020412,  0.0020612,  0.0020807,
     &  0.0020998,  0.0021185,  0.0021368,  0.0021090,  0.0021725,
     &  0.0021898,  0.0022068,  0.0022235,  0.0022399,  0.0022561,
     &  0.0022720,  0.0022876,  0.0023029,  0.0023181,  0.0023330,
     &  0.0023477,  0.0023621,  0.0023764,  0.0023904,  0.0024042,
     &  0.0024179,  0.0024314,  0.0024446,  0.0024577,  0.0024707,
     &  0.0024834,  0.0024960,  0.0025085,  0.0025208,  0.0025329,
     &  0.0025449,  0.0025568/
      data ((coa(i,j),i=1,62),j= 73, 73)/
     &  0.0011902,  0.0012672,  0.0013347,  0.0013952,  0.0014502,
     &  0.0015008,  0.0015478,  0.0015919,  0.0016334,  0.0016727,
     &  0.0017101,  0.0017457,  0.0017799,  0.0018126,  0.0018442,
     &  0.0018746,  0.0019039,  0.0019323,  0.0019598,  0.0019865,
     &  0.0020124,  0.0020376,  0.0020621,  0.0020859,  0.0021092,
     &  0.0021319,  0.0021083,  0.0021757,  0.0021969,  0.0022176,
     &  0.0022379,  0.0022577,  0.0022772,  0.0022962,  0.0023149,
     &  0.0023333,  0.0023513,  0.0023690,  0.0023863,  0.0024034,
     &  0.0024202,  0.0024366,  0.0024529,  0.0024688,  0.0024845,
     &  0.0025000,  0.0025152,  0.0025302,  0.0025450,  0.0025595,
     &  0.0025739,  0.0025880,  0.0026019,  0.0026157,  0.0026293,
     &  0.0026426,  0.0026557,  0.0026689,  0.0026817,  0.0026944,
     &  0.0027070,  0.0027194/
      data ((coa(i,j),i=1,62),j= 74, 74)/
     &  0.0012749,  0.0013552,  0.0014259,  0.0014895,  0.0015475,
     &  0.0016010,  0.0016509,  0.0016977,  0.0017418,  0.0017836,
     &  0.0018234,  0.0018614,  0.0018978,  0.0019328,  0.0019664,
     &  0.0019987,  0.0020300,  0.0020602,  0.0020895,  0.0021179,
     &  0.0021454,  0.0021722,  0.0021982,  0.0022235,  0.0022482,
     &  0.0022723,  0.0022958,  0.0023187,  0.0023411,  0.0023630,
     &  0.0023844,  0.0024054,  0.0024259,  0.0024460,  0.0024658,
     &  0.0024851,  0.0025040,  0.0025226,  0.0025409,  0.0025588,
     &  0.0025764,  0.0025937,  0.0026107,  0.0026275,  0.0026439,
     &  0.0026601,  0.0026760,  0.0026917,  0.0027071,  0.0027223,
     &  0.0027373,  0.0027520,  0.0027665,  0.0027808,  0.0027950,
     &  0.0028089,  0.0028226,  0.0028361,  0.0028495,  0.0028627,
     &  0.0028757,  0.0028885/
      data ((coa(i,j),i=1,62),j= 75, 75)/
     &  0.0013631,  0.0014474,  0.0015220,  0.0015892,  0.0016507,
     &  0.0017076,  0.0017607,  0.0018105,  0.0018575,  0.0019021,
     &  0.0019445,  0.0019850,  0.0020238,  0.0020610,  0.0020967,
     &  0.0021312,  0.0021186,  0.0021965,  0.0022276,  0.0022577,
     &  0.0022868,  0.0023152,  0.0023427,  0.0023695,  0.0023956,
     &  0.0024210,  0.0024457,  0.0024699,  0.0024935,  0.0025165,
     &  0.0025390,  0.0025610,  0.0025826,  0.0026037,  0.0026243,
     &  0.0026445,  0.0026643,  0.0026838,  0.0027028,  0.0027215,
     &  0.0027399,  0.0027579,  0.0027756,  0.0027930,  0.0028101,
     &  0.0028269,  0.0028434,  0.0028596,  0.0028756,  0.0028913,
     &  0.0029068,  0.0029220,  0.0029370,  0.0029518,  0.0029664,
     &  0.0029807,  0.0029949,  0.0030088,  0.0030225,  0.0030361,
     &  0.0030494,  0.0030626/
      data ((coa(i,j),i=1,62),j= 76, 76)/
     &  0.0014557,  0.0015446,  0.0016236,  0.0016950,  0.0017605,
     &  0.0018211,  0.0018777,  0.0019308,  0.0019809,  0.0020284,
     &  0.0020736,  0.0021167,  0.0021121,  0.0021973,  0.0022353,
     &  0.0022718,  0.0023069,  0.0023409,  0.0023737,  0.0024055,
     &  0.0024362,  0.0024661,  0.0024950,  0.0025232,  0.0025506,
     &  0.0025772,  0.0026031,  0.0026284,  0.0026531,  0.0026771,
     &  0.0027006,  0.0027235,  0.0027460,  0.0027679,  0.0027893,
     &  0.0028103,  0.0028309,  0.0028510,  0.0028707,  0.0028900,
     &  0.0029090,  0.0029276,  0.0029458,  0.0029637,  0.0029813,
     &  0.0029986,  0.0030156,  0.0030322,  0.0030486,  0.0030647,
     &  0.0030806,  0.0030962,  0.0031115,  0.0031266,  0.0031414,
     &  0.0031560,  0.0031704,  0.0031846,  0.0031986,  0.0032123,
     &  0.0032259,  0.0032392/
      data ((coa(i,j),i=1,62),j= 77, 77)/
     &  0.0015532,  0.0016476,  0.0017317,  0.0018078,  0.0018775,
     &  0.0019422,  0.0020024,  0.0020590,  0.0021123,  0.0021169,
     &  0.0022106,  0.0022563,  0.0022999,  0.0023416,  0.0023817,
     &  0.0024202,  0.0024572,  0.0024929,  0.0025273,  0.0025607,
     &  0.0025929,  0.0026241,  0.0026543,  0.0026837,  0.0027122,
     &  0.0027399,  0.0027668,  0.0027931,  0.0028186,  0.0028435,
     &  0.0028678,  0.0028915,  0.0029146,  0.0029372,  0.0029592,
     &  0.0029808,  0.0030019,  0.0030225,  0.0030427,  0.0030625,
     &  0.0030819,  0.0031009,  0.0031195,  0.0031377,  0.0031556,
     &  0.0031732,  0.0031904,  0.0032073,  0.0032239,  0.0032402,
     &  0.0032563,  0.0032720,  0.0032875,  0.0033027,  0.0033177,
     &  0.0033324,  0.0033468,  0.0033611,  0.0033751,  0.0033889,
     &  0.0034025,  0.0034159/
      data ((coa(i,j),i=1,62),j= 78, 78)/
     &  0.0016566,  0.0017571,  0.0018467,  0.0019278,  0.0020021,
     &  0.0020708,  0.0021349,  0.0021949,  0.0022514,  0.0023047,
     &  0.0023554,  0.0024035,  0.0024494,  0.0024932,  0.0025352,
     &  0.0025755,  0.0026142,  0.0026515,  0.0026874,  0.0027220,
     &  0.0027555,  0.0027878,  0.0028191,  0.0028495,  0.0028789,
     &  0.0029075,  0.0029352,  0.0029621,  0.0029883,  0.0030138,
     &  0.0030387,  0.0030629,  0.0030864,  0.0031094,  0.0031319,
     &  0.0031538,  0.0031752,  0.0031961,  0.0032166,  0.0032365,
     &  0.0032561,  0.0032752,  0.0032940,  0.0033123,  0.0033303,
     &  0.0033479,  0.0033652,  0.0033821,  0.0033988,  0.0034151,
     &  0.0034310,  0.0034467,  0.0034622,  0.0034773,  0.0034922,
     &  0.0035068,  0.0035211,  0.0035353,  0.0035491,  0.0035628,
     &  0.0035762,  0.0035894/
      data ((coa(i,j),i=1,62),j= 79, 79)/
     &  0.0017664,  0.0018736,  0.0019689,  0.0020552,  0.0021342,
     &  0.0022071,  0.0022749,  0.0023383,  0.0023978,  0.0024539,
     &  0.0025070,  0.0025574,  0.0026054,  0.0026511,  0.0026948,
     &  0.0027366,  0.0027767,  0.0028153,  0.0028523,  0.0028880,
     &  0.0029224,  0.0029556,  0.0029877,  0.0030187,  0.0030487,
     &  0.0030778,  0.0031060,  0.0031334,  0.0031599,  0.0031857,
     &  0.0032108,  0.0032352,  0.0032590,  0.0032821,  0.0033047,
     &  0.0033267,  0.0033481,  0.0033690,  0.0033894,  0.0034094,
     &  0.0034289,  0.0034479,  0.0034665,  0.0034847,  0.0035025,
     &  0.0035200,  0.0035371,  0.0035538,  0.0035702,  0.0035862,
     &  0.0036020,  0.0036174,  0.0036325,  0.0036474,  0.0036620,
     &  0.0036763,  0.0036903,  0.0037041,  0.0037177,  0.0037310,
     &  0.0037441,  0.0037569/
      data ((coa(i,j),i=1,62),j= 80, 80)/
     &  0.0018832,  0.0019973,  0.0020987,  0.0021902,  0.0022737,
     &  0.0023505,  0.0024220,  0.0024885,  0.0025508,  0.0026093,
     &  0.0026646,  0.0027169,  0.0027666,  0.0028138,  0.0028589,
     &  0.0029018,  0.0029430,  0.0029824,  0.0030202,  0.0030565,
     &  0.0030915,  0.0031252,  0.0031576,  0.0031890,  0.0032192,
     &  0.0032485,  0.0032768,  0.0033042,  0.0033308,  0.0033566,
     &  0.0033816,  0.0034059,  0.0034295,  0.0034525,  0.0034748,
     &  0.0034966,  0.0035177,  0.0035384,  0.0035585,  0.0035781,
     &  0.0035972,  0.0036159,  0.0036341,  0.0036520,  0.0036694,
     &  0.0036864,  0.0037031,  0.0037193,  0.0037353,  0.0037509,
     &  0.0037662,  0.0037811,  0.0037958,  0.0038102,  0.0038243,
     &  0.0038381,  0.0038517,  0.0038650,  0.0038780,  0.0038908,
     &  0.0039034,  0.0039158/
      data ((coa(i,j),i=1,62),j= 81, 81)/
     &  0.0020072,  0.0021282,  0.0022356,  0.0023321,  0.0024200,
     &  0.0025006,  0.0025751,  0.0026443,  0.0027089,  0.0027695,
     &  0.0028265,  0.0028803,  0.0029311,  0.0029794,  0.0030252,
     &  0.0030689,  0.0031106,  0.0031504,  0.0031886,  0.0032251,
     &  0.0032602,  0.0032939,  0.0033263,  0.0033575,  0.0033876,
     &  0.0034166,  0.0034447,  0.0034718,  0.0034980,  0.0035234,
     &  0.0035480,  0.0035719,  0.0035950,  0.0036175,  0.0036393,
     &  0.0036605,  0.0036811,  0.0037012,  0.0037207,  0.0037398,
     &  0.0037583,  0.0037764,  0.0037940,  0.0038112,  0.0038280,
     &  0.0038444,  0.0038604,  0.0038761,  0.0038914,  0.0039063,
     &  0.0039210,  0.0039353,  0.0039494,  0.0039631,  0.0039766,
     &  0.0039898,  0.0040027,  0.0040154,  0.0040278,  0.0040400,
     &  0.0040520,  0.0040638/
      data ((coa(i,j),i=1,62),j= 82, 82)/
     &  0.0021381,  0.0022661,  0.0023791,  0.0024803,  0.0025719,
     &  0.0026557,  0.0027328,  0.0028041,  0.0028705,  0.0029325,
     &  0.0029905,  0.0030451,  0.0030966,  0.0031453,  0.0031914,
     &  0.0032353,  0.0032769,  0.0033166,  0.0033545,  0.0033908,
     &  0.0034255,  0.0034588,  0.0034907,  0.0035214,  0.0035509,
     &  0.0035793,  0.0036067,  0.0036331,  0.0036586,  0.0036833,
     &  0.0037071,  0.0037302,  0.0037526,  0.0037743,  0.0037953,
     &  0.0038157,  0.0038356,  0.0038548,  0.0038736,  0.0038916,
     &  0.0039095,  0.0039268,  0.0039436,  0.0039600,  0.0039761,
     &  0.0039917,  0.0040069,  0.0040218,  0.0040364,  0.0040506,
     &  0.0040645,  0.0040781,  0.0040914,  0.0041044,  0.0041172,
     &  0.0041296,  0.0041419,  0.0041539,  0.0041656,  0.0041772,
     &  0.0041885,  0.0041996/
      data ((coa(i,j),i=1,62),j= 83, 83)/
     &  0.0022756,  0.0024100,  0.0025280,  0.0026332,  0.0027280,
     &  0.0028142,  0.0028931,  0.0029658,  0.0030331,  0.0030957,
     &  0.0031541,  0.0032089,  0.0032603,  0.0033088,  0.0033545,
     &  0.0033978,  0.0034389,  0.0034780,  0.0035151,  0.0035506,
     &  0.0035844,  0.0036168,  0.0036478,  0.0036775,  0.0037061,
     &  0.0037335,  0.0037599,  0.0037853,  0.0038098,  0.0038335,
     &  0.0038563,  0.0038784,  0.0038998,  0.0039205,  0.0039405,
     &  0.0039599,  0.0039788,  0.0039971,  0.0040149,  0.0040322,
     &  0.0040490,  0.0040653,  0.0040813,  0.0040968,  0.0041119,
     &  0.0041267,  0.0041411,  0.0041552,  0.0041689,  0.0041823,
     &  0.0041954,  0.0042082,  0.0042208,  0.0042331,  0.0042451,
     &  0.0042569,  0.0042684,  0.0042797,  0.0042908,  0.0043017,
     &  0.0043123,  0.0043228/
      data ((coa(i,j),i=1,62),j= 84, 84)/
     &  0.0024185,  0.0025586,  0.0026808,  0.0027890,  0.0028859,
     &  0.0029735,  0.0030533,  0.0031264,  0.0031938,  0.0032562,
     &  0.0033142,  0.0033683,  0.0034190,  0.0034665,  0.0035113,
     &  0.0035535,  0.0035934,  0.0036313,  0.0036672,  0.0037014,
     &  0.0037340,  0.0037651,  0.0037948,  0.0038232,  0.0038505,
     &  0.0038767,  0.0039018,  0.0039260,  0.0039493,  0.0039717,
     &  0.0039934,  0.0040143,  0.0040345,  0.0040540,  0.0040730,
     &  0.0040913,  0.0041091,  0.0041264,  0.0041432,  0.0041595,
     &  0.0041753,  0.0041907,  0.0042057,  0.0042204,  0.0042346,
     &  0.0042485,  0.0042621,  0.0042754,  0.0042883,  0.0043009,
     &  0.0043133,  0.0043254,  0.0043372,  0.0043488,  0.0043601,
     &  0.0043712,  0.0043821,  0.0043928,  0.0044032,  0.0044135,
     &  0.0044236,  0.0044335/
      data ((coa(i,j),i=1,62),j= 85, 85)/
     &  0.0025653,  0.0027099,  0.0028350,  0.0029450,  0.0030428,
     &  0.0031307,  0.0032102,  0.0032828,  0.0033493,  0.0034106,
     &  0.0034673,  0.0035200,  0.0035692,  0.0036152,  0.0036583,
     &  0.0036990,  0.0037373,  0.0037735,  0.0038078,  0.0038404,
     &  0.0038714,  0.0039009,  0.0039291,  0.0039560,  0.0039818,
     &  0.0040065,  0.0040303,  0.0040531,  0.0040751,  0.0040962,
     &  0.0041166,  0.0041363,  0.0041553,  0.0041738,  0.0041916,
     &  0.0042089,  0.0042256,  0.0042419,  0.0042577,  0.0042730,
     &  0.0042879,  0.0043025,  0.0043166,  0.0043304,  0.0043439,
     &  0.0043570,  0.0043698,  0.0043823,  0.0043945,  0.0044064,
     &  0.0044181,  0.0044296,  0.0044407,  0.0044517,  0.0044624,
     &  0.0044730,  0.0044833,  0.0044934,  0.0045033,  0.0045130,
     &  0.0045226,  0.0045320/
      data ((coa(i,j),i=1,62),j= 86, 86)/
     &  0.0027124,  0.0028597,  0.0029861,  0.0030963,  0.0031936,
     &  0.0032805,  0.0033586,  0.0034295,  0.0034941,  0.0035534,
     &  0.0036081,  0.0036587,  0.0037058,  0.0037497,  0.0037908,
     &  0.0038294,  0.0038657,  0.0039000,  0.0039324,  0.0039631,
     &  0.0039923,  0.0040201,  0.0040467,  0.0040720,  0.0040963,
     &  0.0041195,  0.0041418,  0.0041633,  0.0041839,  0.0042038,
     &  0.0042230,  0.0042415,  0.0042594,  0.0042768,  0.0042935,
     &  0.0043098,  0.0043256,  0.0043409,  0.0043558,  0.0043703,
     &  0.0043844,  0.0043982,  0.0044115,  0.0044246,  0.0044373,
     &  0.0044497,  0.0044619,  0.0044737,  0.0044853,  0.0044966,
     &  0.0045077,  0.0045186,  0.0045292,  0.0045397,  0.0045499,
     &  0.0045599,  0.0045697,  0.0045793,  0.0045888,  0.0045981,
     &  0.0046072,  0.0046162/
      data ((coa(i,j),i=1,62),j= 87, 87)/
     &  0.0028481,  0.0029956,  0.0031209,  0.0032292,  0.0033241,
     &  0.0034083,  0.0034836,  0.0035515,  0.0036132,  0.0036696,
     &  0.0037214,  0.0037692,  0.0038136,  0.0038548,  0.0038934,
     &  0.0039296,  0.0039636,  0.0039956,  0.0040260,  0.0040547,
     &  0.0040820,  0.0041080,  0.0041328,  0.0041565,  0.0041792,
     &  0.0042010,  0.0042219,  0.0042419,  0.0042613,  0.0042800,
     &  0.0042980,  0.0043154,  0.0043322,  0.0043485,  0.0043643,
     &  0.0043796,  0.0043945,  0.0044090,  0.0044231,  0.0044367,
     &  0.0044501,  0.0044631,  0.0044757,  0.0044881,  0.0045001,
     &  0.0045119,  0.0045234,  0.0045347,  0.0045457,  0.0045564,
     &  0.0045670,  0.0045773,  0.0045874,  0.0045973,  0.0046070,
     &  0.0046166,  0.0046259,  0.0046351,  0.0046441,  0.0046530,
     &  0.0046616,  0.0046702/
      data ((coa(i,j),i=1,62),j= 88, 88)/
     &  0.0029341,  0.0030768,  0.0031968,  0.0032997,  0.0033892,
     &  0.0034681,  0.0035383,  0.0036014,  0.0036584,  0.0037104,
     &  0.0037581,  0.0038020,  0.0038427,  0.0038805,  0.0039159,
     &  0.0039490,  0.0039801,  0.0040095,  0.0040373,  0.0040637,
     &  0.0040888,  0.0041127,  0.0041355,  0.0041573,  0.0041782,
     &  0.0041983,  0.0042175,  0.0042361,  0.0042540,  0.0042713,
     &  0.0042880,  0.0043041,  0.0043197,  0.0043349,  0.0043495,
     &  0.0043638,  0.0043777,  0.0043911,  0.0044042,  0.0044170,
     &  0.0044294,  0.0044416,  0.0044534,  0.0044649,  0.0044762,
     &  0.0044872,  0.0044980,  0.0045085,  0.0045188,  0.0045289,
     &  0.0045387,  0.0045484,  0.0045579,  0.0045672,  0.0045763,
     &  0.0045852,  0.0045940,  0.0046026,  0.0046110,  0.0046193,
     &  0.0046275,  0.0046355/
      data ((coa(i,j),i=1,62),j= 89, 89)/
     &  0.0029122,  0.0030427,  0.0031513,  0.0032438,  0.0033237,
     &  0.0033938,  0.0034559,  0.0035116,  0.0035619,  0.0036076,
     &  0.0036495,  0.0036882,  0.0037238,  0.0037573,  0.0037884,
     &  0.0038176,  0.0038451,  0.0038711,  0.0038957,  0.0039191,
     &  0.0039413,  0.0039626,  0.0039829,  0.0040023,  0.0040209,
     &  0.0040389,  0.0040561,  0.0040727,  0.0040887,  0.0041042,
     &  0.0041192,  0.0041337,  0.0041477,  0.0041613,  0.0041745,
     &  0.0041874,  0.0041998,  0.0042120,  0.0042238,  0.0042353,
     &  0.0042465,  0.0042574,  0.0042681,  0.0042785,  0.0042887,
     &  0.0042986,  0.0043084,  0.0043179,  0.0043272,  0.0043363,
     &  0.0043452,  0.0043539,  0.0043625,  0.0043709,  0.0043791,
     &  0.0043872,  0.0043951,  0.0044029,  0.0044105,  0.0044180,
     &  0.0044254,  0.0044326/
      data ((coa(i,j),i=1,62),j= 90, 90)/
     &  0.0027405,  0.0028512,  0.0029426,  0.0030199,  0.0030864,
     &  0.0031447,  0.0031962,  0.0032424,  0.0032841,  0.0033221,
     &  0.0033569,  0.0033891,  0.0034190,  0.0034468,  0.0034729,
     &  0.0034974,  0.0035206,  0.0035424,  0.0035632,  0.0035830,
     &  0.0036018,  0.0036198,  0.0036370,  0.0036535,  0.0036694,
     &  0.0036847,  0.0036993,  0.0037135,  0.0037272,  0.0037404,
     &  0.0037532,  0.0037656,  0.0037776,  0.0037892,  0.0038005,
     &  0.0038115,  0.0038222,  0.0038326,  0.0038427,  0.0038526,
     &  0.0038622,  0.0038716,  0.0038808,  0.0038897,  0.0038984,
     &  0.0039070,  0.0039153,  0.0039235,  0.0039314,  0.0039393,
     &  0.0039469,  0.0039544,  0.0039618,  0.0039690,  0.0039761,
     &  0.0039830,  0.0039898,  0.0039965,  0.0040030,  0.0040095,
     &  0.0040158,  0.0040220/
      data ((coa(i,j),i=1,62),j= 91, 91)/
     &  0.0024633,  0.0025514,  0.0026239,  0.0026851,  0.0027377,
     &  0.0027838,  0.0028247,  0.0028613,  0.0028946,  0.0029249,
     &  0.0029529,  0.0029787,  0.0030028,  0.0030253,  0.0030464,
     &  0.0030663,  0.0030851,  0.0031029,  0.0031199,  0.0031360,
     &  0.0031514,  0.0031661,  0.0031803,  0.0031938,  0.0032068,
     &  0.0032194,  0.0032314,  0.0032431,  0.0032544,  0.0032652,
     &  0.0032758,  0.0032860,  0.0032959,  0.0033055,  0.0033148,
     &  0.0033239,  0.0033327,  0.0033413,  0.0033497,  0.0033578,
     &  0.0033658,  0.0033735,  0.0033811,  0.0033885,  0.0033957,
     &  0.0034028,  0.0034097,  0.0034164,  0.0034230,  0.0034295,
     &  0.0034359,  0.0034421,  0.0034482,  0.0034541,  0.0034600,
     &  0.0034657,  0.0034714,  0.0034769,  0.0034823,  0.0034877,
     &  0.0034929,  0.0034981/
      data ((coa(i,j),i=1,62),j= 92, 92)/
     &  0.0021142,  0.0022278,  0.0022837,  0.0023309,  0.0023717,
     &  0.0024075,  0.0024394,  0.0024681,  0.0024943,  0.0025182,
     &  0.0025404,  0.0025609,  0.0025801,  0.0025980,  0.0026149,
     &  0.0026308,  0.0026459,  0.0026602,  0.0026738,  0.0026868,
     &  0.0026992,  0.0027111,  0.0027225,  0.0027334,  0.0027439,
     &  0.0027541,  0.0027638,  0.0027733,  0.0027824,  0.0027912,
     &  0.0027997,  0.0028080,  0.0028160,  0.0028238,  0.0028314,
     &  0.0028387,  0.0028459,  0.0028529,  0.0028597,  0.0028663,
     &  0.0028727,  0.0028791,  0.0028852,  0.0028912,  0.0028971,
     &  0.0029028,  0.0029084,  0.0029139,  0.0029193,  0.0029246,
     &  0.0029297,  0.0029348,  0.0029398,  0.0029446,  0.0029494,
     &  0.0029541,  0.0029587,  0.0029632,  0.0029676,  0.0029720,
     &  0.0029761,  0.0029805/
      data ((coa(i,j),i=1,62),j= 93, 93)/
     &  0.0018726,  0.0019238,  0.0019660,  0.0020019,  0.0020331,
     &  0.0020606,  0.0020852,  0.0021074,  0.0021278,  0.0021464,
     &  0.0021179,  0.0021798,  0.0021948,  0.0022089,  0.0022221,
     &  0.0022347,  0.0022466,  0.0022578,  0.0022686,  0.0022789,
     &  0.0022887,  0.0022981,  0.0023071,  0.0023158,  0.0023241,
     &  0.0023321,  0.0023399,  0.0023474,  0.0023546,  0.0023616,
     &  0.0023684,  0.0023750,  0.0023814,  0.0023876,  0.0023936,
     &  0.0023995,  0.0024052,  0.0024108,  0.0024162,  0.0024215,
     &  0.0024266,  0.0024317,  0.0024366,  0.0024414,  0.0024461,
     &  0.0024507,  0.0024552,  0.0024596,  0.0024639,  0.0024681,
     &  0.0024722,  0.0024763,  0.0024802,  0.0024841,  0.0024880,
     &  0.0024917,  0.0024954,  0.0024990,  0.0025026,  0.0025060,
     &  0.0025095,  0.0025129/
      data ((coa(i,j),i=1,62),j= 94, 94)/
     &  0.0016337,  0.0016718,  0.0017033,  0.0017303,  0.0017537,
     &  0.0017745,  0.0017931,  0.0018100,  0.0018254,  0.0018397,
     &  0.0018529,  0.0018651,  0.0018766,  0.0018874,  0.0018976,
     &  0.0019073,  0.0019164,  0.0019251,  0.0019334,  0.0019413,
     &  0.0019489,  0.0019562,  0.0019631,  0.0019698,  0.0019763,
     &  0.0019825,  0.0019885,  0.0019944,  0.0020000,  0.0020054,
     &  0.0020107,  0.0020158,  0.0020208,  0.0020256,  0.0020303,
     &  0.0020349,  0.0020394,  0.0020437,  0.0020479,  0.0020521,
     &  0.0020561,  0.0020600,  0.0020639,  0.0020676,  0.0020713,
     &  0.0020749,  0.0020784,  0.0020819,  0.0020852,  0.0020886,
     &  0.0020918,  0.0020950,  0.0020981,  0.0021011,  0.0021041,
     &  0.0021071,  0.0021100,  0.0021128,  0.0021156,  0.0021183,
     &  0.0021210,  0.0021236/
      data ((coa(i,j),i=1,62),j= 95, 95)/
     &  0.0014740,  0.0015024,  0.0015259,  0.0015460,  0.0015636,
     &  0.0015791,  0.0015931,  0.0016058,  0.0016174,  0.0016282,
     &  0.0016381,  0.0016474,  0.0016561,  0.0016643,  0.0016720,
     &  0.0016793,  0.0016863,  0.0016929,  0.0016992,  0.0017052,
     &  0.0017110,  0.0017165,  0.0017219,  0.0017270,  0.0017319,
     &  0.0017367,  0.0017413,  0.0017458,  0.0017501,  0.0017543,
     &  0.0017584,  0.0017623,  0.0017661,  0.0017699,  0.0017735,
     &  0.0017770,  0.0017804,  0.0017838,  0.0017870,  0.0017902,
     &  0.0017933,  0.0017964,  0.0017993,  0.0018023,  0.0018051,
     &  0.0018079,  0.0018106,  0.0018132,  0.0018158,  0.0018184,
     &  0.0018209,  0.0018233,  0.0018258,  0.0018281,  0.0018304,
     &  0.0018327,  0.0018349,  0.0018371,  0.0018393,  0.0018414,
     &  0.0018434,  0.0018455/
      data ((coa(i,j),i=1,62),j= 96, 96)/
     &  0.0013895,  0.0014110,  0.0014289,  0.0014441,  0.0014574,
     &  0.0014692,  0.0014798,  0.0014894,  0.0014982,  0.0015064,
     &  0.0015139,  0.0015210,  0.0015277,  0.0015338,  0.0015398,
     &  0.0015454,  0.0015508,  0.0015558,  0.0015607,  0.0015653,
     &  0.0015698,  0.0015740,  0.0015782,  0.0015821,  0.0015859,
     &  0.0015896,  0.0015932,  0.0015966,  0.0016000,  0.0016032,
     &  0.0016064,  0.0016094,  0.0016124,  0.0016153,  0.0016181,
     &  0.0016208,  0.0016235,  0.0016261,  0.0016286,  0.0016311,
     &  0.0016335,  0.0016358,  0.0016381,  0.0016404,  0.0016426,
     &  0.0016447,  0.0016468,  0.0016489,  0.0016509,  0.0016529,
     &  0.0016548,  0.0016567,  0.0016586,  0.0016604,  0.0016622,
     &  0.0016639,  0.0016657,  0.0016673,  0.0016690,  0.0016706,
     &  0.0016722,  0.0016738/
      data ((coa(i,j),i=1,62),j= 97, 97)/
     &  0.0013502,  0.0013669,  0.0013807,  0.0013924,  0.0014027,
     &  0.0014118,  0.0014200,  0.0014274,  0.0014343,  0.0014406,
     &  0.0014465,  0.0014520,  0.0014571,  0.0014620,  0.0014666,
     &  0.0014710,  0.0014751,  0.0014791,  0.0014829,  0.0014865,
     &  0.0014900,  0.0014933,  0.0014966,  0.0014997,  0.0015027,
     &  0.0015055,  0.0015083,  0.0015109,  0.0015136,  0.0015162,
     &  0.0015186,  0.0015210,  0.0015234,  0.0015256,  0.0015278,
     &  0.0015299,  0.0015320,  0.0015340,  0.0015360,  0.0015380,
     &  0.0015398,  0.0015417,  0.0015435,  0.0015452,  0.0015469,
     &  0.0015486,  0.0015503,  0.0015519,  0.0015534,  0.0015550,
     &  0.0015565,  0.0015580,  0.0015594,  0.0015608,  0.0015622,
     &  0.0015636,  0.0015649,  0.0015663,  0.0015676,  0.0015688,
     &  0.0015701,  0.0015713/
      data ((coa(i,j),i=1,62),j= 98, 98)/
     &  0.0013341,  0.0013476,  0.0013588,  0.0013683,  0.0013766,
     &  0.0013840,  0.0013907,  0.0013967,  0.0014023,  0.0014074,
     &  0.0014122,  0.0014167,  0.0014209,  0.0014248,  0.0014286,
     &  0.0014321,  0.0014355,  0.0014387,  0.0014418,  0.0014447,
     &  0.0014476,  0.0014503,  0.0014529,  0.0014554,  0.0014578,
     &  0.0014602,  0.0014624,  0.0014646,  0.0014667,  0.0014688,
     &  0.0014708,  0.0014727,  0.0014746,  0.0014764,  0.0014782,
     &  0.0014799,  0.0014816,  0.0014833,  0.0014849,  0.0014864,
     &  0.0014879,  0.0014894,  0.0014909,  0.0014923,  0.0014937,
     &  0.0014950,  0.0014964,  0.0014977,  0.0014989,  0.0015002,
     &  0.0015014,  0.0015026,  0.0015038,  0.0015049,  0.0015060,
     &  0.0015071,  0.0015082,  0.0015093,  0.0015103,  0.0015114,
     &  0.0015124,  0.0015134/
      data ((coa(i,j),i=1,62),j= 99, 99)/
     &  0.0013255,  0.0013373,  0.0013470,  0.0013554,  0.0013626,
     &  0.0013691,  0.0013749,  0.0013803,  0.0013851,  0.0013896,
     &  0.0013938,  0.0013977,  0.0014014,  0.0014049,  0.0014082,
     &  0.0014113,  0.0014142,  0.0014171,  0.0014197,  0.0014223,
     &  0.0014248,  0.0014272,  0.0014294,  0.0014316,  0.0014337,
     &  0.0014358,  0.0014378,  0.0014397,  0.0014415,  0.0014433,
     &  0.0014450,  0.0014467,  0.0014483,  0.0014499,  0.0014515,
     &  0.0014530,  0.0014544,  0.0014559,  0.0014573,  0.0014586,
     &  0.0014599,  0.0014612,  0.0014625,  0.0014637,  0.0014649,
     &  0.0014661,  0.0014672,  0.0014684,  0.0014695,  0.0014705,
     &  0.0014716,  0.0014726,  0.0014736,  0.0014746,  0.0014756,
     &  0.0014766,  0.0014775,  0.0014784,  0.0014793,  0.0014802,
     &  0.0014811,  0.0014820/
      data ((coa(i,j),i=1,62),j=100,100)/
     &  0.0013126,  0.0013234,  0.0013324,  0.0013401,  0.0013469,
     &  0.0013529,  0.0013583,  0.0013632,  0.0013677,  0.0013719,
     &  0.0013758,  0.0013795,  0.0013829,  0.0013861,  0.0013891,
     &  0.0013920,  0.0013947,  0.0013974,  0.0013998,  0.0014022,
     &  0.0014045,  0.0014067,  0.0014088,  0.0014108,  0.0014127,
     &  0.0014146,  0.0014164,  0.0014181,  0.0014198,  0.0014215,
     &  0.0014230,  0.0014246,  0.0014261,  0.0014275,  0.0014289,
     &  0.0014303,  0.0014316,  0.0014329,  0.0014341,  0.0014354,
     &  0.0014366,  0.0014377,  0.0014389,  0.0014400,  0.0014411,
     &  0.0014421,  0.0014432,  0.0014442,  0.0014452,  0.0014462,
     &  0.0014471,  0.0014480,  0.0014490,  0.0014499,  0.0014507,
     &  0.0014516,  0.0014525,  0.0014533,  0.0014541,  0.0014549,
     &  0.0014557,  0.0014565/
      data ((coa(i,j),i=1,62),j=101,101)/
     &  0.0012882,  0.0012983,  0.0013066,  0.0013138,  0.0013202,
     &  0.0013258,  0.0013309,  0.0013355,  0.0013398,  0.0013437,
     &  0.0013473,  0.0013507,  0.0013539,  0.0013569,  0.0013598,
     &  0.0013625,  0.0013650,  0.0013674,  0.0013697,  0.0013719,
     &  0.0013740,  0.0013760,  0.0013780,  0.0013798,  0.0013816,
     &  0.0013833,  0.0013850,  0.0013865,  0.0013881,  0.0013896,
     &  0.0013910,  0.0013924,  0.0013938,  0.0013951,  0.0013963,
     &  0.0013976,  0.0013988,  0.0013999,  0.0014011,  0.0014022,
     &  0.0014033,  0.0014043,  0.0014054,  0.0014064,  0.0014073,
     &  0.0014083,  0.0014092,  0.0014101,  0.0014110,  0.0014119,
     &  0.0014128,  0.0014136,  0.0014145,  0.0014153,  0.0014161,
     &  0.0014168,  0.0014176,  0.0014183,  0.0014191,  0.0014198,
     &  0.0014205,  0.0014212/


	u0d=0.5 !! DIFFUSE ANGLE
	swhd=0.0 ; swhu=0.0
	swud=0.0 ; swuu=0.0
	so2d=0.0 ; so2u=0.0
	
	do k=1,np
	ta(k)= .5*(tl(k)+tl(k+1))
	wa(k)= .5*(ql(k)+ql(k+1))

	 dp(k)=pl(k+1)-pl(k)
	 
	 scal(k) = dp(k) *(.5*(pl(k)+pl(k+1) )/300.)**.8
         wh(k)=1.02*wa(k)*scal(k)*(1.+0.00135*(ta(k)-240.))+1.e-11
          swhd(k+1)=swhd(k)+wh(k)
	enddo
	swhu(np+1)=swhd(np+1) ! SAVE column ammount to begin 2nd pass
!	swhu(np+1)= 0.0
	do k=np,1,-1
	 swhu(k)=swhu(k+1)+wh(k)
	enddo

c-----scale co2 amounts following Eq. (3.5) with f=1.
c     unit is (cm-atm)stp.
c     the constant 789 equals (1000/980)*(44/28.97)*(22400/44)

       do k=1,np
         so2d(k+1)=so2d(k)+ 789.*(co2/u0)*scal(k)+1.e-11
       enddo
	so2u(np+1)=so2d(np+1)! SAVE column ammount to begin 2nd pass
!	so2u(np+1)=0.0
       do k=np,1,-1
         so2u(k)=so2u(k+1)+ 789.*(co2/u0d)*scal(k)+1.e-11
       enddo

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!      
!! Band 10
c-----for co2 absorption in Band 10 where absorption due to
c     water vapor and co2 are both moderate

        u1=-3.0
        du=0.15

        w1=-4.0
        dw=0.15

c-----so2 and swh are the co2 and water vapor amounts integrated
c     from the top of the atmosphere
!-dn
      do k= 2, np+1
        swud(k)=log10( so2d(k) )
        swhd(k)=log10( swhd(k)/u0 )
      enddo

c-----df is the updated flux reduction given by the second term on the
c     right-hand-side of Eq. (3.24)

       call rflx(np,swud,u1,du,nu,swhd,w1,dw,nw,cah,df)
 
!-up	 
      do k= 1, np+1
        swuu(k)=log10( so2u(k) )
        swhu(k)=log10( swhu(k)/u0d )
      enddo

       call rflx(np,swuu,u1,du,nu,swhu,w1,dw,nw,cah,dfu)

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!! Band 11
c-----for co2 absorption in Band 11 where the co2 absorption has
c     a large impact on the heating of middle atmosphere.

        u1=0.000250
        du=0.000050

        w1=-2.0
        dw=0.05

c-----co2 mixing ratio is independent of space
c-----swh is the logarithm of presure
!-dn
	do k= 1, np+1
         swud(k)= co2/u0
	 swhd(k)=log10( pl(k) )
        enddo

c-----df is the updated flux reduction derived from the table given by
c     Eq. (3.19)
	
        call rflx(np,swud,u1,du,nx,swhd,w1,dw,ny,coa,df)
!-up
	do k= 1, np+1
         swuu(k)= co2/u0d
	 swhu(k)=log10( pl(k) )
        enddo

c-----df is the updated flux reduction derived from the table given by
c     Eq. (3.19)
	
        call rflx(np,swuu,u1,du,nx,swhu,w1,dw,ny,coa,dfu)

	df(1)=0.0
	return
	end
!================================================================
      subroutine rflx(np,swc,u1,du,nu,swh,w1,dw,nw,tbl,df)

c*****************************************************************

c-----compute the reduction of clear-sky downward solar flux
c     due to co2 absorption.

      implicit none

c-----input parameters

      integer np,nu,nw
      real u1,du,w1,dw
      real swc(np+1),swh(np+1),tbl(nu,nw)

c-----output (undated) parameter

      real df(np+1)

c-----temporary array

      integer i,k,ic,iw 
      real clog,wlog,dc,dd,x1,x2,y1,y2

c-----table look-up for the reduction of clear-sky solar

         x1=u1-0.5*du
         y1=w1-0.5*dw

      do k=1, np+1

          clog=swc(k)
          wlog=swh(k)
          ic=int( (clog-x1)/du+1.)
          iw=int( (wlog-y1)/dw+1.)
          if(ic.lt.2)ic=2
          if(iw.lt.2)iw=2
          if(ic.gt.nu)ic=nu
          if(iw.gt.nw)iw=nw     
          dc=clog-float(ic-2)*du-u1
          dd=wlog-float(iw-2)*dw-w1   
          x2=tbl(ic-1,iw-1)+(tbl(ic-1,iw)-tbl(ic-1,iw-1))/dw*dd
          y2=x2+(tbl(ic,iw-1)-tbl(ic-1,iw-1))/du*dc
          df(k)=df(k)+y2
  
      enddo  

      return
      end
!----------------------------------------------------------------
!----------------------------------------------------------------
	subroutine chou_o2(np,pl,u0,df,dfu)
!! np = # of layers
!! np+1 = # of levels
!! pl(np+1) = Pressure levels (hPa)
!! u0   = Cosine Solar Zenith 
!! df (np+1) = Reduction in flux (fraction) of  downwelling SW

c-----compute pressure-scaled o2 amount following Eq. (3.5) with
c     f=1. unit is (cm-atm)stp.
c     the constant 165.22 equals (1000/980)*23.14%*(22400/32)
c-----compute flux reduction due to oxygen following Eq. (3.18)
c     the constant 0.0633 is the fraction of insolation contained 
c     in the oxygen bands

	real pl(np+1) 
	real dp(np)   , scal(np)
	real  so2d(np+1) , df(np+1)
	real  so2u(np+1) , dfu(np+1)
	
	so2d=0.0
	so2u=0.0
!! WHATS IN HIS PAPER JCLIM Feb90 pg212
	frac=0.02867 !(39.23/1368.)
	c1=-0.00027
!! WHATS IN Chou's CODE 
	frac= 0.0633 ! (86.5944/1368.)
	c1=-0.000145

	do k=1,np
	 dp(k)=pl(k+1)-pl(k)
	 scal(k) = dp(k) *(.5*(pl(k)+pl(k+1) )/300.)**.8
	 so2d(k+1)= so2d(k)+scal(k)* 165.22/u0
	enddo

	so2u(np+1)=so2d(np+1)

	do k=np,1,-1
	 so2u(k)= so2u(k+1)+scal(k)* 165.22/u0
	enddo

       do k= 1, np+1
        df (k)=df (k)+frac*(1.-exp(c1*sqrt( so2d(k) )))
	dfu(k)=dfu(k)+frac*(1.-exp(c1*sqrt( so2u(k) )))
       enddo          

	return
	end
